.\" -*- nroff -*- generated from .Rd format
.BG
.FN box.cox.powers
.TL
Multivariate Unconditional Box-Cox Transformations
.DN
Estimates multivariate unconditional power transformations
to multinormality
by the method of maximum likelihood. The univariate case is
obtained when only one variable is specified.
.IP
Note that this is unconditional Box-Cox. That is, there is
no regression model, and there are no predictors. The object is to
make the distribution of the variable(s) as (multi)normal as possible.
For Box-Cox regression, see the `boxcox' function in the 
`MASS' library.
.IP  
The function estimates the Box-Cox powers, 
x' = (x^p - 1)/p 
for p != 0 and x' = log(x)
for p = 0. Subsequently using ordinary power
transformations (i.e., x^p for p != 0
achieves the same result.
.CS
box.cox.powers(X, start=NULL, ...)

print.box.cox.powers(x, ...)

summary.box.cox.powers(object, digits=4, ...)
.RA
.AG X
a numeric matrix of variables (or a vector for one variable)
to be transformed.
.AG x, object
`box.cox.powers' object.
.OA
.AG start
start values for the power transformation parameters;
if `NULL' (the default), univariate Box-Cox transformations will
be computed and used as the start values.
.AG ...
optional arguments to be passed to the `optim' function.
.AG digits
number of places to round result.
.RT
returns an object of class `box.cox.powers', which may be printed
or summarized.
.SH REFERENCES
Box, G. E. P. and Cox, D. R. (1964)
An analysis of transformations.
JRSS B 26 211-246.
.IP 
Cook, R. D. and Weisberg, S. (1999)
Applied Regression, Including Computing and Graphics. Wiley.
.SA
`box.cox', `box.cox.var', 
`TransformationAxes'
.EX
attach(Prestige)
summary(box.cox.powers(cbind(income, education)))
## Box-Cox Transformations to Multinormality 
## 
##  income education 
##  0.2617    0.4241
## 
## L.R. test, all powers = 0:  7.694   df = 2   p = 0.0213
## L.R. test, all powers = 1:  48.8727   df = 2   p = 0 

plot(income, education)
plot(box.cox(income, .26), box.cox(education, .42))

summary(box.cox.powers(income))
## Box-Cox Transformation to Normality 
## 
##         
##  0.1793
## 
## L.R. test, power = 0:  2.7103   df = 1   p = 0.0997
## L.R. test, power = 1:  47.261   df = 1   p = 0 

qq.plot(income)
qq.plot(income^.18)
.KW multivariate
.KW models
.WR
