.\" -*- nroff -*- generated from .Rd format
.BG
.FN box.tidwell
.TL
Box-Tidwell Transformations
.DN
Computes the Box-Tidwell power transformations of the predictors in a
linear model.
.IP
The maximum-likelihood estimates of the transformation parameters are computed
by Box and Tidwell's (1962) method, which is usually more efficient than using
a general nonlinear least-squares routine for this problem. Score tests for the
transformations are also reported.

.CS
box.tidwell(y, ...)

box.tidwell.formula(formula, other.x=NULL, data=NULL, subset, 
  na.action=options()$na.action, verbose=F, tol=0.001, 
  max.iter=25, ...)

box.tidwell.default(y, x1, x2=NULL, max.iter=25, tol=0.001, 
  verbose=F, ...)
  
print.box.tidwell(x, digits, ...)
.RA
.AG formula
two-sided formula, the right-hand-side of which gives the
predictors to be transformed.
.AG y
response variable.
.AG x1
matrix of predictors to transform.
.OA
.AG other.x
one-sided formula giving the predictors that are not
candidates for transformation, including (e.g.) factors.
.AG data
an optional data frame containing the variables in the model. 
By default the variables are taken from the environment from which 
`box.tidwell' is called.
.AG subset
an optional vector specifying a subset of observations to be used.
.AG na.action
a function that indicates what should happen when the data contain `NA's. 
The default is set by the `na.action' setting of `options'.
.AG verbose
if `TRUE' a record of iterations is printed.
.AG tol
if maximum relative change in coefficients is less than `tol' then
convergence is declared.
.AG max.iter
maximum number of iterations.
.AG x2
matrix of predictors that are not candidates for transformation.
.AG \&...
not for the user.
.AG x
`box.tidwell' object.
.AG digits
number of digits for rounding.
.RT
an object of class `box.tidwell', which is normally just printed.
.SH REFERENCES
Box, G. E. P. and Tidwell, P. W. (1962)
Transformation of the independent variables.
Technometrics 4, 531-550.
.IP 
Fox, J. (1997)
Applied Regression, Linear Models, and Related Methods. Sage.

.EX
box.tidwell(prestige~income+education, ~ poly(women,2), data=Prestige)
##                   income education
## Initial Power   -0.91030   2.24354
## Score Statistic -5.30129   2.40556
## p-value          0.00000   0.01615
## MLE of Power    -0.03777   2.19283
.KW regression
.WR
