options(echo=T)





is.relative<-function(term1, term2, factors) {
    all(!(factors[,term1]&(!factors[,term2])))
    }

relatives<-function(term, names, factors){
    if(length(names)==1) return(NULL)
    which.term<-which(term==names)
    (1:length(names))[-which.term][sapply(names[-which.term], 
        function(term2, term, factors) is.relative(term, term2, factors), term=term, factors=factors)]
    }
    


Anova<-function(mod, ...){
    UseMethod("Anova", mod)
    }

 
 

Anova.lm<-function(mod, error=NULL, type=c("II","III"), ...){
    type<-match.arg(type)
    switch(type,
        II=Anova.II.lm(mod, error, ...),
        III=Anova.III.lm(mod, error, ...))
    }


Anova.aov <- function(mod, ...){
    
    class(mod) <- "lm"
    Anova.lm(mod, ...)
    }

        


Anova.II.lm<-function(mod, error, ...){
    if (!is.null(error)){
        sumry<-summary(error, corr=FALSE)
        s2<-sumry$sigma^2
        error.df<-error$df.residual
        error.SS<-s2*error.df
        }
    SS.term<-function(term, names, fac, mod, sumry, I.p, assign, ...){ 
        subs.term<-assign[[term]]
        relatives<-relatives(term, names, fac)
        subs.relatives<-NULL
        for (relative in relatives) subs.relatives<-c(subs.relatives, assign[[relative]])
        hyp.matrix.1<-I.p[subs.relatives,]
        hyp.matrix.2<-I.p[c(subs.relatives,subs.term),]
        SS1<-if (length(subs.relatives)==0) 0 
            else linear.hypothesis(mod, hyp.matrix.1, summary.model=sumry, ...)$SSH
        SS2<-linear.hypothesis(mod, hyp.matrix.2, summary.model=sumry, ...)$SSH
        SS2-SS1
        }
    fac<-attr(mod$terms, "factors")
    intercept<-has.intercept(mod)
    p<-length(coefficients(mod))
    I.p<-diag(p)
    assign<-mod$assign
    names<-term.names(mod)
    if (intercept) {
        names<-names[-1]
        assign <- assign[-1]
        }
    n.terms<-length(names)
    SS<-rep(0, n.terms+1)
    df<-rep(0, n.terms+1)
    F0<-rep(0, n.terms+1)
    p<-rep(0, n.terms+1)
    sumry<-summary(mod, corr = FALSE)
    SS[n.terms+1]<-if (is.null(error)) sumry$sigma^2*mod$df.residual else error.SS   
    df[n.terms+1]<-if (is.null(error)) mod$df.residual else error.df
    F0[n.terms+1]<-NA
    p[n.terms+1]<-NA
    for (i in 1:n.terms){
        SS[i]<-SS.term(names[i], names, fac, mod, sumry, I.p, assign, ...)
        df[i]<-df.terms(mod, names[i])
        F0[i]<-df[n.terms+1]*SS[i]/(df[i]*SS[n.terms+1])
        p[i]<-1-pf(F0[i],df[i],df[n.terms+1])
        }    
    result<-data.frame(SS, df, F0, p)
    row.names(result)<-c(names,"Residuals")
    names(result)<-c("Sum Sq", "Df", "F value", "Pr(>F)")
    class(result)<-c("anova","data.frame")
    attr(result,"heading")<-c("Anova Table (Type II tests)\n", paste("Response:", response.name(mod)))
    result
    }


        


Anova.III.lm<-function(mod, error, ...){
    if (!is.null(error)){
        sumry<-summary(error, corr=FALSE)
        s2<-sumry$sigma^2
        error.df<-error$df.residual
        error.SS<-s2*error.df
        }
    intercept<-has.intercept(mod)
    p<-length(coefficients(mod))
    I.p<-diag(p)
    Source<-term.names(mod)
    n.terms<-length(Source)
    SS<-rep(0, n.terms+1)
    df<-rep(0, n.terms+1)
    F0<-rep(0, n.terms+1)
    p<-rep(0, n.terms+1)
    assign<-mod$assign
    sumry<-summary(mod, corr = FALSE)
    for (term in 1:n.terms){
        subs<-assign[[term]]
        hyp.matrix<-I.p[subs,]
        test<-if (is.null(error)) linear.hypothesis(mod, hyp.matrix, summary.model=sumry, ...)
            else linear.hypothesis(mod, hyp.matrix, error.SS=error.SS, error.df=error.df, 
                summary.model=sumry, ...)
        SS[term]<-test$SSH
        df[term]<-test$Df[1]
        F0[term]<-test$F0
        p[term]<-test$p
        }
     Source[n.terms+1]<-"Residuals"
     df.res<-if (is.null(error)) mod$df.residual
        else error.df     
     s2<-sumry$sigma^2
     SS[n.terms+1]<-if (is.null(error)) s2*df.res
        else error.SS
     df[n.terms+1]<-df.res
     F0[n.terms+1]<-NA
     p[n.terms+1]<-NA
     result<-data.frame(SS, df, F0, p)
     row.names(result)<-Source
     names(result)<-c("Sum Sq", "Df", "F value", "Pr(>F)")
     class(result)<-c("anova","data.frame")
     attr(result,"heading")<-c("Anova Table (Type III tests)\n", paste("Response:", response.name(mod)))
     result
     }


    
    
Anova.glm<-function(mod, type=c("II","III"), test.statistic=c("LR", "Wald", "F"), 
    error=NULL, error.estimate=c("pearson", "dispersion", "deviance")){
    
    type<-match.arg(type)
    test.statistic<-match.arg(test.statistic)
    error.estimate<-match.arg(error.estimate)
    switch(type,
        II=switch(test.statistic,
            LR=Anova.II.LR.glm(mod),
            Wald=Anova.II.Wald.glm(mod),
            F=Anova.II.F.glm(mod, error, error.estimate)),
        III=switch(test.statistic,
            LR=Anova.III.LR.glm(mod),
            Wald=Anova.III.Wald.glm(mod),
            F=Anova.III.F.glm(mod, error, error.estimate)))
    }

    
        
        
            
        
Anova.III.Wald.glm<-function(mod){
    
    intercept<-has.intercept(mod)
    p<-length(coefficients(mod))
    I.p<-diag(p)
    Source<-term.names(mod)
    n.terms<-length(Source)
    Wald<-rep(0, n.terms)
    df<-rep(0, n.terms)
    p<-rep(0, n.terms)
    assign<-attr(model.matrix(mod),"assign")
    sumry<-summary(mod, corr=FALSE)
    for (term in 1:n.terms){
        subs<-which(assign==term-intercept)
        hyp.matrix<-I.p[subs,]
        test<-linear.hypothesis(mod, hyp.matrix, summary.model=sumry)
        Wald[term]<-test$ChiSquare
        df[term]<-test$Df
        p[term]<-test$p
        }
     result<-data.frame(Wald, df, p)
     row.names(result)<-Source
     names(result)<-c("Wald Chisq","Df","Pr(>Chisq)")
     class(result)<-c("anova","data.frame")
     attr(result,"heading")<-c("Anova Table (Type III tests)\n", paste("Response:", response.name(mod)))
     result
     }
     
            

Anova.III.LR.glm<-function(mod){
    Source<-if (has.intercept(mod)) term.names(mod)[-1]
        else term.names(mod)
    n.terms<-length(Source)
    LR<-rep(0, n.terms)
    df<-rep(0, n.terms)
    p<-rep(0, n.terms)
    dispersion<-summary(mod, corr = FALSE)$dispersion
    deviance<-deviance(mod)/dispersion
    for (term in 1:n.terms){
        mod.1<-drop1(mod, scope=
            eval(parse(text=paste("~",Source[term]))))
        LR[term]<-(mod.1$Deviance[2]/dispersion)-deviance
        df[term]<-mod.1$Df[2]
        p[term]<-1-pchisq(LR[term], df[term])
        }
     result<-data.frame(LR, df, p)
     row.names(result)<-Source
     names(result)<-c("LR Chisq","Df","Pr(>Chisq)")
     class(result)<-c("anova","data.frame")
     attr(result,"heading")<-c("Anova Table (Type III tests)\n", paste("Response:", response.name(mod)))
     result
     }

            

Anova.III.F.glm<-function(mod, error, error.estimate){
    
    if (is.null(error)) error<-mod
    df.res <- df.residual(error)
    error.SS<-switch(error.estimate,
        pearson=sum(residuals(error, "pearson")^2),
        dispersion=df.res*summary(error, corr = FALSE)$dispersion,
        deviance=deviance(error))
    Source<-if (has.intercept(mod)) term.names(mod)[-1]
        else term.names(mod)
    n.terms<-length(Source)
    p <- df <- f <- SS <-rep(0, n.terms+1)
    f[n.terms+1] <- p[n.terms+1] <- NA
    df[n.terms+1]<-df.res
    SS[n.terms+1]<-error.SS
    dispersion<-error.SS/df.res
    deviance<-deviance(mod)
    for (term in 1:n.terms){
        mod.1<-drop1(mod, scope=
            eval(parse(text=paste("~",Source[term]))))
        df[term]<-mod.1$Df[2]
        SS[term]<-mod.1$Deviance[2] - deviance
        f[term]<-(SS[term]/df[term])/dispersion
        p[term]<-1-pf(f[term], df[term], df.res)
        }
     result<-data.frame(SS, df, f, p)
     row.names(result)<-c(Source, "Residuals")
     names(result)<-c("SS", "Df", "F", "Pr(>F)")
     class(result)<-c("anova","data.frame")
     attr(result,"heading")<-c("Anova Table (Type III tests)\n", paste("Response:", response.name(mod)))
     result
     }
     
        
        
            
        
Anova.II.Wald.glm<-function(mod){
    
    chisq.term<-function(term){
        which.term<-which(term==names)
        subs.term<-which(assign==which.term)
        relatives<-relatives(term, names, fac)
        subs.relatives<-NULL
        for (relative in relatives) subs.relatives<-c(subs.relatives, which(assign==relative))
        hyp.matrix.1<-I.p[subs.relatives,]
        hyp.matrix.2<-I.p[c(subs.relatives,subs.term),]
        sumry<-summary(mod, corr=FALSE)
        chisq.1<-if (length(subs.relatives)==0) 0 
            else linear.hypothesis(mod, hyp.matrix.1, summary.model=sumry)$ChiSquare
        chisq.2<-linear.hypothesis(mod, hyp.matrix.2, summary.model=sumry)$ChiSquare
        chisq.2-chisq.1
        }
    fac<-attr(mod$terms, "factors")
    intercept<-has.intercept(mod)
    p<-length(coefficients(mod))
    I.p<-diag(p)
    names<-term.names(mod)
    if (intercept) names<-names[-1]
    assign<-rep(1:length(names), df.terms(mod))
    assign<-if (intercept) c(0,assign) else assign
    n.terms<-length(names)
    Wald<-rep(0, n.terms)
    df<-rep(0, n.terms)
    p<-rep(0, n.terms)
    for (i in 1:n.terms){
        Wald[i]<-chisq.term(names[i])
        df[i]<-df.terms(mod, names[i])
        p[i]<-1-pchisq(Wald[i],df[i])
        }    
    result<-data.frame(Wald, df, p)
    row.names(result)<-names
    names(result)<-c("Wald Chisq","Df","Pr(>Chisq)")
    class(result)<-c("anova","data.frame")
    attr(result,"heading")<-c("Anova Table (Type II tests)\n", paste("Response:", response.name(mod)))
    result
    }

            
            
Anova.II.LR.glm<-function(mod){
    fac<-attr(mod$terms, "factors")
    names<-if (has.intercept(mod)) term.names(mod)[-1]
        else term.names(mod)
    n.terms<-length(names)
    LR<-rep(0, n.terms)
    df<-df.terms(mod)
    p<-rep(0, n.terms)
    dispersion<-summary(mod, corr = FALSE)$dispersion
    for (term in 1:n.terms){
        rels<-names[relatives(names[term], names, fac)]
        mod.1<-update(mod,
            eval(parse(text=paste(".~.-",paste(c(names[term], rels), collapse="-")))))
        dev.1<-deviance(mod.1)
        mod.2<-if (length(rels)==0) mod
            else update(mod,
                    eval(parse(text=paste(".~.-",paste(rels, collapse="-")))))
        dev.2<-deviance(mod.2)
        LR[term]<-(dev.1-dev.2)/dispersion
        p[term]<-1-pchisq(LR[term], df[term])
        }
     result<-data.frame(LR, df, p)
     row.names(result)<-names
     names(result)<-c("LR Chisq","Df","Pr(>Chisq)")
     class(result)<-c("anova","data.frame")
     attr(result,"heading")<-c("Anova Table (Type II tests)\n", paste("Response:", response.name(mod)))
     result
     }


            
            
Anova.II.F.glm<-function(mod, error, error.estimate){
    
    if (is.null(error)) error<-mod
    df.res <- df.residual(error)
    error.SS<-switch(error.estimate,
        pearson=sum(residuals(error, "pearson")^2),
        dispersion=df.res*summary(error, corr = FALSE)$dispersion,
        deviance=deviance(error))
    fac<-attr(mod$terms, "factors")
    names<-if (has.intercept(mod)) term.names(mod)[-1]
        else term.names(mod)
    n.terms<-length(names)
    p <- df <- f <- SS <-rep(0, n.terms+1)
    f[n.terms+1] <- p[n.terms+1] <- NA
    df[n.terms+1]<-df.res
    SS[n.terms+1]<-error.SS
    dispersion<-error.SS/df.res
    f <- p <- rep(0, n.terms+1)
    f[n.terms+1] <- p[n.terms+1] <- NA
    df<-c(df.terms(mod), df.res)
    for (term in 1:n.terms){
        rels<-names[relatives(names[term], names, fac)]
        mod.1<-update(mod,
            eval(parse(text=paste(".~.-",paste(c(names[term], rels), collapse="-")))))
        dev.1<-deviance(mod.1)
        mod.2<-if (length(rels)==0) mod
            else update(mod,
                    eval(parse(text=paste(".~.-",paste(rels, collapse="-")))))
        dev.2<-deviance(mod.2)
        SS[term]<-dev.1 - dev.2
        f[term]<-SS[term]/(dispersion*df[term])
        p[term]<-1-pf(f[term], df[term], df.res)
        }
     result<-data.frame(SS, df, f, p)
     row.names(result)<-c(names, "Residuals")
     names(result)<-c("SS","Df","F","Pr(>F)")
     class(result)<-c("anova","data.frame")
     attr(result,"heading")<-c("Anova Table (Type II tests)\n", paste("Response:", response.name(mod)))
     result
     }



Ask<-function(arg, fun, ...){ 
    fun<-fun
    repeat{   
        value<-readline(paste("Enter",deparse(substitute(arg)),": "))
        
        if (value == "") return(NULL)
        
        eval(parse(text=paste("fun(",deparse(substitute(arg)),"=",value,",...)")))
        }
    }
 


avp<-function(...) av.plots(...)

av.plots<-function(model, variable, ask=missing(variable), one.page=!ask, ...){
    
    if (!missing(variable)){
        var<-if (is.character(variable) & 1==length(variable)) variable
            else deparse(substitute(variable))
        av.plot(model, var, ...)
        }
    else {
        vars<-colnames(model.matrix(model))
        if (ask) {
            repeat{
                selection<-menu(vars)
                if (selection==0) break
                else var<-vars[selection]
                av.plot(model, var, ...)
                }
            }
        else {
            if (one.page){
                save.mfrow <- par(mfrow=mfrow(length(vars)))
                
                
                
                
                on.exit(par(save.mfrow))
                
                }
            for (var in vars) av.plot(model, var, ...)
            }
        }
    }


av.plot<-function (model, ...) {
    UseMethod("av.plot")
    }


av.plot.lm<-function(model, variable, labels=names(residuals(model)[!is.na(residuals(model))]), 
    identify.points=T, las=par('las'), col=2, pch=1, lwd=2, main="Added-Variable Plot"){

    
    variable<-if (is.character(variable) & 1==length(variable)) variable
        else deparse(substitute(variable))
    mod.mat<-model.matrix(model)
    var.names<-colnames(mod.mat)
    var<-which(variable==var.names)
    if (0==length(var)) stop(paste(variable,"is not a column of the model matrix."))
    response<-response(model)
    response.name<-response.name(model)
    if (is.null(weights(model))) wt<-rep(1, length(response))
        else wt<-weights(model)
    res<-lsfit(mod.mat[,-var], cbind(mod.mat[,var], response), wt=wt,    
        intercept=F)$residuals
    plot(res[,1], res[,2], xlab=paste(var.names[var],"| others"), 
        ylab=paste(response.name," | others"), main=main, las=las, col=col, pch=pch)
    abline(lsfit(res[,1], res[,2], wt=wt), col=col, lwd=lwd)
    if (identify.points) identify(res[,1], res[,2], labels)
    }


av.plot.glm<-function(model, variable, labels=names(residuals(model)[!is.na(residuals(model))]), 
    identify.points=T, las=par('las'), col=2, pch=1, lwd=2, main="Added-Variable Plot",
    type=c("Wang", "Weisberg")){

    
    type<-match.arg(type)
    variable<-if (is.character(variable) & 1==length(variable)) variable
        else deparse(substitute(variable))
    mod.mat<-model.matrix(model)
    var.names<-colnames(mod.mat)
    var<-which(variable==var.names)
    if (0==length(var)) stop(paste(variable,"is not a column of the model matrix."))
    response<-response(model)
    response.name<-response.name(model)
    wt<-model$prior.weights
    
    if (is.null(wt)) wt <- rep(1, length(response))
    
    mod<-glm(response~mod.mat[,-var]-1, weights=wt, family=family(model))
    res.y<-residuals(mod, type="pearson")
    wt<-if (type=="Wang") wt*na.omit(weights(model)) else wt
    res.x<-lsfit(mod.mat[,-var], mod.mat[,var], wt=wt,    
        intercept=F)$residuals
    plot(res.x, res.y, xlab=paste(var.names[var],"| others"), 
        ylab=paste(response.name," | others"), main=main, las=las, col=col, pch=pch)
    abline(lsfit(res.x, res.y, wt=wt), col=col, lwd=lwd)
    if (identify.points) identify(res.x, res.y, labels)
    }




nice<-function(x, direction=c("round", "down", "up")){
    direction<-match.arg(direction)
    if (length(x)>1) return(sapply(x, nice, direction=direction))
    if (x==0) return(0)
    power.10<-floor(log(abs(x),10))
    lead.digit<-switch(direction,
        round=round(abs(x)/10^power.10),
        down=floor(abs(x)/10^power.10),
        up=ceiling(abs(x)/10^power.10))
    sign(x)*lead.digit*10^power.10
    }






power.axis<-function(power, base=exp(1), side=c("right", "above", "left", "below"), 
    at, grid=F, grid.col=2, grid.lty=3,
    axis.title = "Untransformed Data", cex = 1, las=par("las")) {

    
    side<-if(is.numeric(side)) side 
        else which(match.arg(side)==c("below", "left", "above", "right"))
    axp<-if (side %% 2 == 1) par("xaxp") else par("yaxp")
    ticks<-nice(seq(from=axp[1], to=axp[2], length=axp[3]+1))
    ticks.x<- if (power !=0) nice(ticks[ticks>0]^(1/power)) 
        else nice(log(base)*exp(ticks))
    ticks.x <- if (missing(at)) ticks.x
        else at
    ticks.text <- as.character(ticks.x)
    ticks.trans<-if (power !=0) ticks.x^power else log(ticks.x, base)
    axis(side, labels = ticks.text, at = ticks.trans, las=las)
    if (grid & (side %% 2 == 0)) abline(h=ticks.trans, lty=grid.lty, col=grid.col)
    if (grid & (side %% 2 == 1)) abline(v=ticks.trans, lty=grid.lty, col=grid.col)
    mtext(axis.title, side = side, line = 3, cex = cex)
    }


box.cox.axis<-function(power, side=c("right", "above", "left", "below"), 
    at, grid=F, grid.col=2, grid.lty=3,
    axis.title = "Untransformed Data", cex = 1, las=par("las")) {

    
    inverse.power<-function(x,p){
        if (p==0) exp(x)
        else (1+p*x)^(1/p)
        }
    side<-if(is.numeric(side)) side 
        else which(match.arg(side)==c("below", "left", "above", "right"))
    axp<-if (side %% 2 == 1) par("xaxp") else par("yaxp")
    ticks<-nice(seq(from=axp[1], to=axp[2], length=axp[3]+1))
    ticks.x<- if (power !=0) nice(inverse.power(ticks[ticks>0], power))
        else nice(inverse.power(ticks, 0))
    ticks.x <- if (missing(at)) ticks.x
        else at
    ticks.text <- as.character(ticks.x)
    ticks.trans<-box.cox(ticks.x,power)
    axis(side, labels = ticks.text, at = ticks.trans, las=las)
    if (grid & (side %% 2 == 0)) abline(h=ticks.trans, lty=grid.lty, col=grid.col)
    if (grid & (side %% 2 == 1)) abline(v=ticks.trans, lty=grid.lty, col=grid.col)
    mtext(axis.title, side = side, line = 3, cex = cex)
    }





prob.axis<-function(at, side=c("right", "above", "left", "below"),
    grid=F, grid.lty=3, grid.col=2,
    axis.title = "Probability", interval = 0.1, cex = 1, las=par("las"))

{
    
    side<-if(is.numeric(side)) side 
        else which(match.arg(side)==c("below", "left", "above", "right"))
    logit<-if (side %% 2 == 1) par("usr")[c(1,2)] else par("usr")[c(3,4)]
    fact <- 10^( - (floor(log(interval, 10))))
    p.min <- nice(1/(1 + exp( - logit[1])), direction="down")
    p.max <- nice(1/(1 + exp( - logit[2])), direction="up")
    tick.min <- max(interval, (floor(fact * p.min))/fact)
    tick.max <- min(1 - interval, (ceiling(fact * p.max))/fact)
    ticks.p <- seq(tick.min, tick.max, interval)
    if(p.min <= 0.05) ticks.p <- c(0.05, ticks.p)
    if(p.min <= 0.01) ticks.p <- c(0.01, ticks.p)
    if(p.max >= 0.95) ticks.p <- c(ticks.p, 0.95)
    if(p.max >= 0.99) ticks.p <- c(ticks.p, 0.99)
    ticks.p<-if (missing(at)) ticks.p else at
    ticks.text <- as.character(ticks.p)
    ticks.logit <- log(ticks.p/(1 - ticks.p))
    axis(side, labels = ticks.text, at = ticks.logit, las=las)
    if (grid & (side %% 2 == 0)) abline(h=ticks.logit, lty=grid.lty, col=grid.col)
    if (grid & (side %% 2 == 1)) abline(v=ticks.logit, lty=grid.lty, col=grid.col)
    mtext(axis.title, side = side, line = 3, cex = cex)
}




modified.power<-function(x, lambda, gm){
    if (lambda == 0) log(x)*gm
    else (gm^(1-lambda))*((x^lambda)-1)/lambda
    }
neg.kernel.profile.logL<-function(lambda, X,  gm){
    for (j in 1:ncol(X)){
        X[,j]<-modified.power(X[,j],lambda[j],gm[j])
        }
    (nrow(X)/2)*log(((nrow(X)-1)/nrow(X))*det(var(X)))
    }
univ.neg.kernel.logL <- function(x, lambda, gm){
    x <- modified.power(x, lambda, gm)
    (length(x)/2)*log(((length(x)-1)/length(x))*var(x))
    }
    
box.cox.powers<-function(X, start=NULL, ...){
    X<-as.matrix(X)
    if(any(X<=0)) stop("All values must be > 0")
    gm<-apply(X, 2, function(x) exp(mean(log(x))))
    if (is.null(start)) {
        start <- rep(1, ncol(X))
        for (j in 1:ncol(X)){
            res<- optimize(
                f = function(lambda, gm, x) univ.neg.kernel.logL(x=x, lambda=lambda, gm=gm),
                lower=-20, upper=+20, gm=gm[j], x=X[,j])
            start[j] <- res$minimum
            }
        }
    res<-nlminb(start, neg.kernel.profile.logL, X=X, gm=gm, ...)
    result<-list()
    result$start<-start
    result$criterion<-res$objective
    result$names<-colnames(X)
    result$lambda<-res$parameters
    result$LR0<-2*(neg.kernel.profile.logL(X=X, lambda=rep(0,ncol(X)), gm=gm)-res$objective)
    result$LR1<-2*(neg.kernel.profile.logL(X=X, lambda=rep(1,ncol(X)), gm=gm)-res$objective)
    result$return.code<-!(res$message == "FALSE")
    if(!result$return.code) 
        warning(paste("Convergence failure: return code =",
            result$return.code))
    class(result)<-"box.cox.powers"
    result
    }
      
summary.box.cox.powers<-function(object, round.digits=4){
    one<-1==length(object$lambda)
    cat(paste("Box-Cox", (if(one) "Transformation to Normality" else "Transformations to Multinormality"),"\n\n"))
    lambda<-object$lambda
    df<-length(lambda)
    names(lambda) <- if (!one) object$names else ""
    print(round(lambda,round.digits))
    cat(paste("\nL.R. test,", (if(one) "power" else "all powers"), "= 0: ",round(object$LR0,round.digits),"  df =",df,
        "  p =",round(1-pchisq(object$LR0,df),round.digits)))
    cat(paste("\nL.R. test,", (if(one) "power" else "all powers"), "= 1: ",round(object$LR1,round.digits),"  df =",df,
        "  p =",round(1-pchisq(object$LR1,df),round.digits),"\n"))
    invisible(object)
    }

print.box.cox.powers <- function(x){
    lambda <- x$lambda
    names(lambda) <- x$names
    print(lambda)
    invisible(x)
    }



bc<-function(x,p) box.cox(x,p)

box.cox<-function(x,p, start=0){
    
    min<-min(x, na.rm=T)
    s<-if (missing(start) & (min <= 0)) nice(-min +.05*diff(quantile(x,c(.25,.75), na.rm=T)), "up")
        else start
    if (missing(start) & s != 0) warning(paste("start = ", s, "added to data prior to transformation"))
    x<-x+s
    if (p==0) log(x)
        else (x^p-1)/p
    }


box.cox.var<-function(y) {
    geo.mean<-exp(mean(log(y),na.rm=T))
    y*(log(y/geo.mean) - 1)
    }


box.tidwell<-function(y, ...){
    UseMethod("box.tidwell")
    }

box.tidwell.formula<-function(formula, other.x=NULL, data=NULL, subset, na.action=options()$na.action, 
    verbose=F, tol=.001, max.iter=25, ...) {
    m <- match.call(expand.dots = FALSE)
    if (is.matrix(eval(m$data, sys.frame(sys.parent())))) 
        m$data <- as.data.frame(data)
    m$formula<-if (is.null(other.x)) formula
        else as.formula(paste(formula[2], "~", formula[3], "+", other.x[2]))
    m$max.iter<-m$tol<-m$verbose<-m$family<-m$other.x <- m$... <- NULL
    m[[1]] <- as.name("model.frame")
    mf <- eval(m, sys.frame(sys.parent()))
    response <- attr(attr(mf, "terms"), "response")
    if (!response) stop(paste("No response variable in model."))
    X1<-model.matrix(formula, data=mf)[,-1]
    X2<-if (is.null(other.x)) NULL
        else model.matrix(other.x, data=mf)[,-1]
    
    
    
    
    y <- model.extract(mf, "response")
    
    box.tidwell.default(y, X1, X2, max.iter=max.iter, tol=tol, verbose=verbose, ...)
    }

box.tidwell.default<-function(y, x1, x2=NULL, max.iter=25, tol=.001, verbose=F, ...) {
    
    x1<-as.matrix(x1)
    var.names<-if(is.null(colnames(x1))) 1:ncol(x1) else colnames(x1)
    k.x1<-length(var.names)
    x.log.x<-x1*log(x1)
    mod.1<-lm(y~cbind(x1, x2), ...)
    mod.2<-lm(y~cbind(x.log.x, x1, x2), ...)
    sumry<-summary(mod.2)
    seb<-sqrt(Var(mod.2, diagonal=T))
    t.vals<-((coefficients(mod.2))/seb)[2:(1+k.x1)]
    initial<-powers<-1+coefficients(mod.2)[2:(1+k.x1)]/coefficients(mod.1)[2:(1+k.x1)]
    pvalues<-2*(1-pnorm(abs(t.vals)))
    iter<-0
    last.powers<-1
    while ( (max(abs((powers-last.powers)/(powers+tol))) > tol) &
        (iter <= max.iter) ) {
        iter<-iter+1
        x1.p<-x1^matrix(powers, nrow=nrow(x1), ncol=ncol(x1), byrow=T)
        x.log.x<-x1.p*log(x1.p)
        mod.1<-lm(y~cbind(x1.p, x2), ...)
        mod.2<-lm(y~cbind(x.log.x, x1.p, x2), ...)
        last.powers<-powers
        powers<-powers * 
            (1+coefficients(mod.2)[2:(1+k.x1)]/coefficients(mod.1)[2:(1+k.x1)])
        if (verbose) cat(" iter =", iter, "    powers =", powers, "\n")
        }
    if (iter > max.iter) warning("maximum iterations exceeded")
    result<-rbind(initial,t.vals, pvalues, powers)
    rownames(result)<-c("Initial Power","Score Statistic","p-value","MLE of Power")
    colnames(result)<-names(powers)<-var.names
    result<-list(result=result, iterations=iter)
    class(result)<-"box.tidwell"
    result
    }

print.box.tidwell<-function(x, digits=5){ 
    
    print(round(x$result, digits))
    cat("\niterations = ", x$iterations,"\n")
    }
  


ceres.plots<-function(model, variable, ask=missing(variable), one.page=!ask, span=.5, ...){
    
    if(!is.null(class(model$na.action)) && 
        class(model$na.action) == 'exclude') class(model$na.action) <- 'omit'
    if (!missing(variable)){
        var<-if (is.character(variable) & 1==length(variable)) variable
            else deparse(substitute(variable))
        ceres.plot(model, var, ...)
        }
    else {
        vars<-predictor.names(model)
        vars<-if (is.null(model$contrasts)) vars
            else vars[sapply(model$contrasts[vars], is.null)]
        if (0==length(vars)) stop("No covariates to plot.")
        if (any(attr(terms(model),"order")>1)) {
            stop("ceres plots not available for models with interactions.")
            }
        if (ask) {
            repeat{
                selection<-menu(c(paste("Change span = ",span),vars))
                if (selection==0) break
                if (selection==1) {
                    span<-eval(parse(text=readline(prompt="span: ")))
                    if ((!is.numeric(span)) || length(span)>1 || span<0
                        || span>1) stop("Span must be between 0 and 1")
                    }
                else {
                    var<-vars[selection-1]
                    ceres.plot(model, var, span=span)
                    }
                }
            }
        else {
            if (one.page){
                save.mfrow <- par(mfrow=mfrow(length(vars)))
                on.exit(par(mfrow=save.mfrow))
                }
            for (var in vars){ 
                 ceres.plot(model, var, span=span, ...)
                 }
            }
        }
    }


ceres.plot<-function (model, ...) {
    UseMethod("ceres.plot")
    }






ceres.plot.lm<-function(model, variable, line=T, smooth=T, span=.5, iter, 
    las=par('las'), col=2, pch=1, lwd=2, main="Ceres Plot"){

    
    
    if(!is.null(class(model$na.action)) && 
        class(model$na.action) == 'exclude') class(model$na.action) <- 'omit'
    if (missing(iter)){
        iter<-if(("glm"==class(model)[1]) &&
                 ("gaussian"!=as.character(family(model))[1]))
                0
                else 3
            }    
    
    
    
    var<-if (is.character(variable) & 1==length(variable)) variable
        else deparse(substitute(variable))
    mod.mat<-model.matrix(model)
    obs<-names(residuals(model))
    all.obs<-if (is.null(model$call$data)) obs else row.names(eval(model$call$data))
    xx<-rep(NA, length(all.obs))
    names(xx)<-all.obs
    vars<-predictor.names(model)
    if (is.na(match(var, vars))) stop(paste(var,"is not in the model."))
    if (!is.null(model$contrasts[[var]])) stop(paste(var,"is a factor."))
    vars<-vars[-match(var,vars)]
    if (any(attr(terms(model),"order")>1)) {
        stop("ceres plot not available for models with interactions.")
        }
    .x<-xvars<-NULL
    for (xvar in vars){
        if (is.null(model$contrasts[[xvar]])){
            xvars<-c(xvars,xvar)
            xx[obs]<-fitted.values(loess(as.formula(paste("mod.mat[,'",xvar,"']~mod.mat[,'",var,"']",sep=""))))
            .x<-cbind(.x, xx)
            }
        }
    if (is.null(xvars)) stop("There are no covariates.")
    n.x<-length(xvars)
    mf<-model.frame(model)
    rownames(.x)<-all.obs
    mf$.x<-.x[obs,]
    aug.model<-update(model, .~.+.x, data=mf)
    
    
    
    
    aug.mod.mat<-model.matrix(aug.model, data=mf)
    
    coef<-coefficients(aug.model)
    k<-length(coef)
    posn<-k:(k-n.x+1)
    
    
    
    
    
    partial.res<-residuals.glm(aug.model, "working")+aug.mod.mat[,var]*coef[var] +
        aug.mod.mat[,posn] %*% as.matrix(coef[posn])
    
    plot(mod.mat[,var], partial.res, xlab=var, col=col, pch=pch,
        ylab=paste("CERES Residual(",response.name(model),")", sep=""),
        main=main, las=las)
    if (line) abline(lm(partial.res~mod.mat[,var]), lty=2, lwd=lwd, col=col)
    if (smooth) {
        lines(lowess(mod.mat[,var], partial.res, iter=iter, f=span), lwd=lwd, col=col)
        }
    }                    

ceres.plot.glm<-function(model, ...){
  
  ceres.plot.lm(model, ...)
  }


crp<-function(...) cr.plots(...)

cr.plots<-function(model, variable, ask=missing(variable), one.page=!ask, span=.5, ...){
    
    if(!is.null(class(model$na.action)) && 
        class(model$na.action) == 'exclude') class(model$na.action) <- 'omit'
    if (!missing(variable)){
        var<-if (is.character(variable) & 1==length(variable)) variable
            else deparse(substitute(variable))
        cr.plot(model, var, ...)
        }
    else {
        vars<-predictor.names(model)
        if (0==length(vars)) stop("No covariates to plot.")
        else if (ask) {
            repeat{
                selection<-menu(c(paste("Change span = ",span),vars))
                if (selection==0) break
                if (selection==1) {
                    span<-eval(parse(text=readline(prompt="span: ")))
                    if ((!is.numeric(span)) || length(span)>1 || span<0
                        || span>1) stop("Span must be between 0 and 1")
                    }
                else {
                    var<-vars[selection-1]
                    cr.plot(model, var, span=span, ...)
                    }
                }
            }
        else {
            if (one.page){
                save.mfrow <- par(mfrow=mfrow(length(vars)))
                
                
                
                
                on.exit(par(save.mfrow))
                
                    }
            for (var in vars){ 
                 cr.plot(model, var, span=span, ...)
                }
            }
        }
    }
    
    
cr.plot<-function (model, ...) {
    UseMethod("cr.plot")
    }



cr.plot.lm<-function(model, variable, order=1, line=T, smooth=T,
    iter, span=.5, las=par('las'), col=2, pch=1, lwd=2,
    main="Component+Residual Plot") {
    
    
    if(!is.null(class(model$na.action)) && 
        class(model$na.action) == 'exclude') class(model$na.action) <- 'omit'
    var<-if (is.character(variable) & 1==length(variable)) variable
        else deparse(substitute(variable))
    vars<-predictor.names(model)
    if (is.na(match(var, vars))) stop(paste(var,"is not in the model."))
    if (any(attr(terms(model),"order")>1)) {
        stop("C+R plots not available for models with interactions.")
        }
    if (!is.null(model$contrasts[[var]])){
        partial.res<-predict(model, type="terms", terms=var) + residuals.glm(model,"working")
        .x<-model.frame(model)[,var]
        boxplot(split(partial.res, .x), xlab=var,
            ylab=paste("Component+Residual(", response.name(model),")", sep=""),
            main=main)
        return(invisible())
        }
    if (missing(iter)){
        iter<-if(("glm"==class(model)[1]) &&
                 ("gaussian"!=as.character(family(model))[1]))
                0
                else 3
            }    
    .x<-if (df.terms(model, var)>1) predict(model, type="terms", term=var)
        else model.matrix(model)[,var]
    if (order==1){          
        mod.mat<-model.matrix(model)
        coef<-coefficients(model)
        partial.res<-residuals.glm(model, "working")+mod.mat[,var]*coef[var]
        plot(.x, partial.res, xlab=var, 
            ylab=paste("Component+Residual(", response.name(model),")", sep=""),
            las=las, col=col, pch=pch, main=main)
        if (line) abline(lm(partial.res~.x), lty=2, lwd=lwd, col=col)
        if (smooth) {
            lines(lowess(.x, partial.res, iter=iter, f=span), lwd=lwd, col=col)
            }
        }
        else {
            if (df.terms(model, var)>1) 
               stop(paste("Order", order, "C+R plot not available for a term with > 1 df:", var))
            aug.model<-update(model, 
                as.formula(paste(".~.-",var,"+poly(",var,",",order,")")), na.action=na.omit)
            aug.mod.mat<-model.matrix(aug.model)
            coef<-coefficients(aug.model)
            cols <- length(coef) - 0:(order - 1)
            partial.res<-residuals.glm(aug.model, "working")+aug.mod.mat[,cols] %*% coef[cols]
            plot(.x, partial.res, xlab=var, 
                ylab=paste("Component+Residual(", response.name(model),")", sep=""),
                las=las, col=col, pch=pch, main=main)
            if (line) abline(lm(partial.res~.x), lty=2, lwd=lwd, col=col)
            if (smooth) {
                lines(lowess(.x, partial.res, iter=iter, f=span), lwd=lwd, col=col)
                }
            }          
    }


cr.plot.glm<-function(model, ...){
  
  cr.plot.lm(model, ...)
 }

Adler <- read.table("Adler.txt")
Angell <- read.table("Angell.txt")      
Anscombe <- read.table("Anscombe.txt")    
Baumann <- read.table("Baumann.txt")     
Bfox <- read.table("Bfox.txt")        
Can.pop <- read.table("Can.pop.txt")
Chile <- read.table("Chile.txt")       
Chirot <- read.table("Chirot.txt")      
Cowles <- read.table("Cowles.txt")      
Davis <- read.table("Davis.txt")       
DavisThin <- read.table("DavisThin.txt")
Duncan <- read.table("Duncan.txt")      
Ericksen <- read.table("Ericksen.txt")    
Florida <- read.table("Florida.txt")     
Freedman <- read.table("Freedman.txt")    
Friendly <- read.table("Friendly.txt")
Ginzberg <- read.table("Ginzberg.txt")    
Greene <- read.table("Greene.txt")      
Guyer <- read.table("Guyer.txt")       
Hartnagel <- read.table("Hartnagel.txt")   
Leinhardt <- read.table("Leinhardt.txt")
Mandel <- read.table("Mandel.txt")      
Migration <- read.table("Migration.txt")   
Moore <- read.table("Moore.txt")       
Mroz <- read.table("Mroz.txt")        
Ornstein <- read.table("Ornstein.txt")
Prestige <- read.table("Prestige.txt")    
Quartet <- read.table("Quartet.txt")     
Robey <- read.table("Robey.txt")       
Sahlins <- read.table("Sahlins.txt")     
SLID <- read.table("SLID.txt")
UN <- read.table("UN.txt")          
US.pop <- read.table("US.pop.txt")      
Vocab <- read.table("Vocab.txt")       
Womenlf <- read.table("Womenlf.txt")     



influence.glm<-function(model) 
{
    b <- coefficients(model)
    p <- length(b)
    n <- length(response(model))
    e <- as.vector(na.omit(residuals(model, type="deviance"))) 
    pear.res <- as.vector(na.omit(residuals(model, type="pearson")))
    res <- c(list(names=names(e), dev.res=e, pear.res=pear.res),
        lm.influence(model)[c("hat", "coefficients", "sigma")])
    res$coefficients <- matrix(b, n, p, byrow=T) - res$coefficients
    if (is.null(model$na.action)) return(res)
    else res <- naresid(model$na.action, cbind(res$dev.res, res$pear.res, 
        res$hat, res$sigma, res$coefficients))
    list(names=rownames(res),dev.res=res[,1], pear.res=res[,2], hat=res[,3], sigma=res[,4], 
        coefficients=res[,-(1:4)])
}

weighted.residuals <- function(model){
    wts <- weights(model)
    res <- residuals(model)
    if (is.null(wts)) res
        else sqrt(wts)*res
    }

variable.names <- function(model) names(coefficients(model))

influence.lm<-function(model) 
{
    b <- coefficients(model)
    p <- length(b)
    n <- length(response(model))
    e <- na.omit(weighted.residuals(model))[,1]
    res <- c(list(names=names(e), wt.res = e),
            lm.influence(model)[c("hat", "coefficients", "sigma")])
    res$coefficients <- matrix(b, n, p, byrow=T) - res$coefficients
    if (is.null(model$na.action)) return(res)
    else res <- naresid(model$na.action, 
        cbind(res$wt.res, res$hat, res$sigma, res$coefficients))
    list(names=rownames(res),wt.res=res[,1], hat=res[,2], sigma=res[,3], 
        coefficients=res[,-(1:3)])
}


hatvalues<-function(model, ...){
    UseMethod("hatvalues")
    }
    
hatvalues.lm<-function(model, infl=influence(model), names=infl$names){
    hat<-infl$hat
    names(hat)<-names
    hat
    }

rstudent<-function(model, ...){
    UseMethod("rstudent")
    }

rstudent.lm<-function(model, infl=influence(model), names=infl$names){
    rstud<-infl$wt.res/(infl$sigma*(1 - infl$hat)^.5)
    names(rstud)<-names
    rstud
    }


rstudent.glm<-function(model, infl=influence(model), names=infl$names){
    rstud<-sign(infl$dev)*sqrt(infl$dev.res^2 + (infl$hat * infl$pear.res^2)/(1 - infl$hat))
    rstud<-if (any(family(model)$family['name'] == c("binomial", "poisson"))) rstud
            else rstud/infl$sigma
    names(rstud)<-names
    rstud
    }


influence<-function(model, ...){
    UseMethod("influence")
    }

cookd<-function(model, ...){
    UseMethod("cookd")
    }
    
cookd.lm<-function(model, infl=influence(model), sumry=summary(model), names=infl$names){
    sigma<-sumry$sigma
    df<-model$rank
    res<-infl$wt.res/(sigma*(1 - infl$hat)^.5)
    cookd<-(res^2 * infl$hat)/(df*(1 - infl$hat))
    names(cookd)<-names
    cookd
    }

cookd.glm<-function(model, infl=influence(model), sumry=summary(model), names=infl$names){   
    phi<-sumry$dispersion
    df<-model$rank
    cookd<-(infl$pear.res^2 * infl$hat)/(phi*df*(1 - infl$hat)^2)
    names(cookd)<-names
    cookd
    }
    
dfbeta<-function(model, ...){
    UseMethod("dfbeta")
    }
    
dfbeta.lm<-function(model, infl=influence(model), names=infl$names){
    b<-infl$coefficients
    dimnames(b)<-list(names, variable.names(model))
    b
    }
    
dfbetas<-function(model, ...){
    UseMethod("dfbetas")
    }
        
dfbetas.lm<-function(model, infl=influence(model), sumry=summary(model), names=infl$names){ 
    sb<-(diag(sumry$cov.unscaled))^.5
    b<-dfbeta(model)/(infl$sigma %o% sb)
    dimnames(b)<-list(names, variable.names(model))
    b    
    }




durbin.watson <- function(model, ...){
  UseMethod("durbin.watson")
  }

durbin.watson.lm <- function(model, max.lag=1, simulate=TRUE, reps=1000, 
    method=c("resample","normal"), 
    alternative=c("two.sided", "positive", "negative")){
    method<-match.arg(method)
    alternative <- if (max.lag == 1) match.arg(alternative)
        else "two.sided"
    residuals<-residuals(model)
    if (any(is.na(residuals))) stop ('residuals include missing values')
    n<-length(residuals)
    r<-dw<-rep(0, max.lag)
    den<-sum(residuals^2)
    for (lag in 1:max.lag){
        dw[lag]<-(sum((residuals[(lag+1):n] - residuals[1:(n-lag)])^2))/den
        r[lag]<-(sum(residuals[(lag+1):n]*residuals[1:(n-lag)]))/den
        }
    if (!simulate){
        result<-list(r=r, dw=dw)
        class(result)<-"durbin.watson"
        result
        }
        else {
            S<-summary(model)$sigma
            X<-model.matrix(model)
            mu<-fitted.values(model)
            Y<-if (method == "resample") 
                matrix(sample(residuals, n*reps, replace=TRUE), n, reps) + matrix(mu, n, reps)
                else matrix(rnorm(n*reps, 0, S), n, reps) + matrix(mu, n, reps)
            E<-residuals(lm(Y~X-1))
            DW<-apply(E, 2, durbin.watson, max.lag=max.lag)
            if (max.lag == 1) DW <- rbind(DW)
            p<-rep(0, max.lag)
            if (alternative == 'two.sided'){
                for (lag in 1:max.lag) {
                    p[lag] <- (sum(dw[lag] < DW[lag,]))/reps
                    p[lag] <- 2*(min(p[lag], 1-p[lag]))
                    }
                }
            else if (alternative == 'positive'){
                for (lag in 1:max.lag) {
                    p[lag] <- (sum(dw[lag] > DW[lag,]))/reps
                    }
                }
            else {
                for (lag in 1:max.lag) {
                    p[lag] <- (sum(dw[lag] < DW[lag,]))/reps
                    }
                }
            result<-list(r=r, dw=dw, p=p, alternative=alternative)
            class(result)<-"durbin.watson"
            result
            }
    }

durbin.watson.default<-function(resids, max.lag=1){
    names(resids)<-NULL
    if ( (!is.vector(resids)) || (!is.numeric(resids)) ) stop("requires vector of residuals")
    if (any(is.na(resids))) stop ('residuals include missing values')
    n<-length(resids)
    dw<-rep(0, max.lag)
    den<-sum(resids^2)
    for (lag in 1:max.lag){
        dw[lag]<-(sum((resids[(lag+1):n] - resids[1:(n-lag)])^2))/den
        }
    dw
    }
    
print.durbin.watson<-function(x, ...){
    max.lag<-length(x$dw)
    result<- if (is.null(x$p)) cbind(lag=1:max.lag,Autocorrelation=x$r, "D-W Statistic"=x$dw)
            else cbind(lag=1:max.lag,Autocorrelation = x$r, "D-W Statistic" = x$dw, 
                  "p-value"= x$p)
    rownames(result)<-rep("", max.lag)
    print(result)
    cat(paste(" Alternative hypothesis: rho", if(max.lag > 1) "[lag]" else "",
        c(" != ", " > ", " < ")[which(x$alternative == c("two.sided", "positive", "negative"))],
        "0\n", sep=""))
    invisible(x)
    }



ellipse<-function(center, shape, radius, center.pch=16, center.cex=1.5, segments=51, add=T, 
        xlab="", ylab="", las=par('las'), col=2, lwd=2, lty=1, ...) {
    names(center) <- NULL

    
    if (! (is.vector(center) && 2==length(center))) stop("center must be a vector of length 2")
    if (! (is.matrix(shape) && all(2==dim(shape)))) stop("shape must be a 2 by 2 matrix")
    angles <- (0:segments)*2*pi/segments 
    unit.circle <- cbind(cos(angles),sin(angles)) 
    ellipse <- t( center + radius * t( unit.circle %*% chol( shape ) ) ) 
    if (add) lines(ellipse, col=col, lwd=lwd, lty=lty, ...) 
    else plot(ellipse, xlab = xlab, ylab = ylab, type ="l", col=col, 
        lwd=lwd, lty=lty, las=las, ... ) 
    if (center.pch) points(center[1],center[2],pch=center.pch,cex=center.cex,col=col)
}


data.ellipse<-function(x, y, levels=c(0.5, 0.9), center.pch=16, center.cex=1.5,
        plot.points=T, add=!plot.points, segments=51, robust=F,
        xlab=deparse(substitute(x)), ylab=deparse(substitute(y)), 
        las=par('las'), col=2, pch=1, lwd=2, lty=1, ...) {

    
    if(missing(y)){
        if(is.matrix(x) && ncol(x)==2) {
            if (missing(xlab)) xlab<-colnames(x)[1]
            if (missing(ylab)) ylab<-colnames(x)[2]
            y<-x[,2]
            x<-x[,1]
            }
        else stop("x and y must be vectors, or x must be a 2 column matrix")
        }
    else {

        names(x) <- names(y) <- NULL

        if(! (is.vector(x) && is.vector(y) && length(x)==length(y)))
            stop("x and y must be vectors of the same length")
        }
    if (plot.points & !add) plot(x, y, xlab=xlab, ylab=ylab, col=col, pch=pch, las=las, ...)
    if (plot.points & add) points(x, y, col=col, pch=pch, ...)
    dfn<-2
    dfd<-length(x)-1
    if (robust) {
        require(MASS)
        v<-cov.trob(cbind(x,y))
        shape<-v$cov
        center<-v$center
        }
    else {
        shape<-var(cbind(x,y))
        center<-c(mean(x), mean(y))
        }
    for (level in levels) {
        radius <- sqrt ( dfn * qf(level, dfn, dfd ))
        ellipse(center, shape, radius, 
            center.pch=center.pch, center.cex=center.cex, segments=segments, 
            col=col, lty=lty, lwd=lwd, ...)
        }
    }

confidence.ellipse<-function (model, ...) {
    UseMethod("confidence.ellipse")
    }
    

confidence.ellipse.lm<-function(model, which.coef, levels=0.95, Scheffe=F, 
        center.pch=16, center.cex=1.5, segments=51, xlab, ylab, 
        las=par('las'), col=2, lwd=2, lty=1, ...){

    
    which.coef<-if(length(coefficients(model)) == 2) c(1,2)
                else{
                    if (missing(which.coef)){
                        if (has.intercept(model)) c(2,3) else c(1,2)
                        } else which.coef
                    }
    coef<-coefficients(model)[which.coef]
    xlab<-if (missing(xlab)) paste(names(coef)[1], "coefficient")
    ylab<-if (missing(ylab)) paste(names(coef)[2], "coefficient")
    dfn<-if (Scheffe) sum(df.terms(model)) else 2
    
    
    
    
    dfd <- model$df.residual
    
    shape<-Var(model)[which.coef, which.coef]
    for (level in rev(sort(levels))){
        radius<-sqrt(dfn*qf(level, dfn, dfd))
        add<-!level==max(levels)
        ellipse(coef, shape, radius, add=add, xlab=xlab, ylab=ylab,
            center.pch=center.pch, center.cex=center.cex, segments=segments, 
            col=col, lwd=lwd, lty=lty, las=las, ...)
        }
    }



confidence.ellipse.glm<-function(model, which.coef, levels=0.95, Scheffe=F, 
        center.pch=16, center.cex=1.5, segments=51, xlab, ylab,
        las=par('las'), col=2, lwd=2, lty=1, ...){

    
    which.coef<-if(length(coefficients(model)) == 2) c(1,2)
                else{
                    if (missing(which.coef)){
                        if (has.intercept(model)) c(2,3) else c(1,2)
                        } else which.coef
                    }
    coef<-coefficients(model)[which.coef]
    xlab<-if (missing(xlab)) paste(names(coef)[1], "coefficient")
    ylab<-if (missing(ylab)) paste(names(coef)[2], "coefficient")
    df<-if (Scheffe) sum(df.terms(model)) else 2
    sumry<-summary(model, corr = FALSE)
    shape<-Var(model)[which.coef, which.coef]
    for (level in rev(sort(levels))){
        radius<-sqrt(qchisq(level, df))
        add<-!level==max(levels)
        ellipse(coef, shape, radius, add=add, xlab=xlab, ylab=ylab,
            center.pch=center.pch, center.cex=center.cex, segments=segments,
            col=col, lwd=lwd, lty=lty, las=las, ...)
        }
    }



df.residual <- function(model) model$df.residual


hccm<-function(model, ...){
    
    UseMethod("hccm")
    }
 
hccm.lm<-function(model, type=c("hc3", "hc0", "hc1", "hc2", "hc4"), ...) {
    
    if (!is.null(weights(model))) stop("requires unweighted lm")
    type <- match.arg(type)
    sumry <- summary(model, corr = FALSE)
    s2 <- sumry$sigma^2
    V <- sumry$cov.unscaled
    if (type == FALSE) return(s2*V)
    e <- na.omit(residuals(model))
    X <- model.matrix(model)
    df.res <- df.residual(model)
    n <- length(e)
    h <- hat(X)
    p <- ncol(X)
    factor<-switch(type,
        hc0=1,
        hc1=df.res/n,
        hc2=1 - h,
        hc3=(1 - h)^2,
        hc4=(1 - h)^pmin(4, n*h/p))
    V %*% t(X) %*% apply(X, 2, "*", (e^2)/factor) %*% V
    }
        
hccm.default<-function(model, ...){
    
    stop("requires an lm object")
    }



palette <- function(n.colors=9) 1:n.colors


leverage.plots<-function(model, term.name, ask=missing(term.name), ...){
    
    if (!missing(term.name)){
        var<-if (is.character(term.name) & 1==length(term.name)) term.name
            else deparse(substitute(term.name))
        leverage.plot(model, term.name, ...)
        }
    else {
        term.names<-term.names(model)
        if (ask) {
            repeat{
                selection<-menu(term.names)
                if (selection==0) break
                else term.name<-term.names[selection]
                leverage.plot(model, term.name, ...)
                }
            }
        else {
            for (term.name in term.names) leverage.plot(model, term.name, ...)
            }
        }
    }


leverage.plot<-function (model, ...) {
    UseMethod("leverage.plot")
    }

leverage.plot.lm<-function(model, term.name, 
    labels=names(residuals(model)[!is.na(residuals(model))]), 
    identify.points=T, las=par('las'), col=2, pch=1, lwd=2, main="Leverage Plot"){
    
    term.name<-if (is.character(term.name) & 1==length(term.name)) term.name
        else deparse(substitute(term.name))
    b<-coefficients(model)
    e<-na.omit(residuals(model))
    p<-length(b)
    I.p<-diag(p)
    term.names<-term.names(model)
    term<-which(term.name==term.names)
    if (0==length(term)) stop(paste(term.name,"is not a term in the model."))
    response.name<-response.name(model)
    intercept<-has.intercept(model)
    assign<-model$assign
    X<-model.matrix(model)
    V<-Var(model)
    wt<-if (is.null(weights(model))) rep(1, length(X[,1]))
        else weights(model)
    
    
    
    
    subs <- assign[[term]]
    
    hypothesis.matrix<-I.p[subs,]
    L<-if (is.null(dim(hypothesis.matrix))) t(hypothesis.matrix)
        else hypothesis.matrix
    u<-inv(L %*% V %*% t(L)) %*% L %*% b
    v.x<-X %*% V %*% t(L) %*% u
    v.y<-v.x + e
    plot(v.x, v.y, xlab=paste(term.names[term],"| others"), 
        ylab=paste(response.name," | others"), main=main,
        las=las, col=col, pch=pch)
    abline(lsfit(v.x, v.y, wt=wt), col=col, lwd=lwd)
    if (identify.points) identify(v.x, v.y, labels)
    }

leverage.plot.glm<-function(model, ...){
    
    stop("leverage plot requires lm object")
    }
  


lht<-function(...) linear.hypothesis(...)

linear.hypothesis<-function (model, ...) {
    UseMethod("linear.hypothesis")
    }


linear.hypothesis.lm<-function(model, hypothesis.matrix, rhs=0, 
        summary.model=summary(model, corr = FALSE),
        white.adjust=F, error.SS, error.df) {
    if (is.aliased(model)) stop ("One or more terms aliased in model.")
    s2<-if (missing(error.SS)) summary.model$sigma^2
        else error.SS/error.df
    V<-if (white.adjust==FALSE) summary.model$cov.unscaled
        else hccm(model, type=white.adjust)/s2
    b<-coefficients(model)
    L<-if (is.null(dim(hypothesis.matrix))) t(hypothesis.matrix)
        else hypothesis.matrix
    q<-nrow(L)
    SSH<-t(L %*% b - rhs) %*% inv(L %*% V %*% t(L)) %*% (L %*% b - rhs)
    F0<-SSH/(q*s2)
    df<-if (missing(error.df)) model$df.residual
        else error.df
    p<-1-pf(F0, q, df)
    result<-list(SSH=SSH[1,1], SSE=s2*df, F0=F0[1,1], Df=c(q, df), p=p)
    class(result)<-"F.test"
    result
    }


linear.hypothesis.glm<-function(model, hypothesis.matrix, rhs=0, 
        summary.model=summary(model, corr = FALSE)) {
    
    if (is.aliased(model)) stop ("One or more terms aliased in model.")
    V<-summary.model$dispersion*summary.model$cov.unscaled
    b<-coefficients(model)
    L<-if (is.null(dim(hypothesis.matrix))) t(hypothesis.matrix)
        else hypothesis.matrix
    q<-nrow(L)
    Wald<-t(L %*% b - rhs) %*% inv(L %*% V %*% t(L)) %*% (L %*% b - rhs)
    p<-1-pchisq(Wald, q)
    result<-list(test="Wald Test",ChiSquare=Wald[1,1], Df=q, p=p[1,1])
    class(result)<-"chisq.test"
    result
    }
 

 print.chisq.test<-function(x){
    title<-if (!is.null(x$test)) x$test else "Chisquare Test"
    cat(title,"\n")
    if (!is.null(x$formula)) cat(x$formula.name, 
        "formula:", as.character(x$formula), "\n")
    cat("Chisquare =", x$ChiSquare,"   Df =", x$Df,
        "    p =", x$p, "\n")
    invisible(x)
    }
 
 print.F.test<-function(x){
    title<-if (!is.null(x$test)) x$test else "F-Test"
    cat(title,"\n")
    if (!is.null(x$formula)) cat(x$formula.name, 
        "formula:", as.character(x$formula), "\n")
    cat("SS =", x$SSH, "    SSE =", x$SSE, "    F =", x$F,
        " Df =", x$Df[1], "and", x$Df[2], "    p =", x$p, "\n")
    invisible(x)
    }


logit<-function(p, percents=max(p, na.rm=T)>1, adjust){   
    if (percents) p<-p/100
    a<-if (missing(adjust)) {
        if (min(p, na.rm=T)==0 | max(p, na.rm=T)==1) .025 else 0
        }
        else adjust
    if (missing(adjust) & a != 0) warning(paste("Proportions remapped to (",
        a,",",1-a,")", sep=""))
    a<-1-2*a
    log((.50+a*(p-.50))/(1-(.50+a*(p-.50))))
    }
    


n.bins<-function(x, rule=c("freedman.diaconis","sturges","scott","simple")){
    
    rule<-match.arg(rule)
    x<-x[!is.na(x)]
    n<-length(x)
    Q<-quantile(x, c(.25,.75))
    X<-range(x)
    result<-switch(rule,
        freedman.diaconis=ceiling(n^(1/3)*(X[2]-X[1])/(2*(Q[2]-Q[1]))),
        sturges=ceiling(log(n,2)+1),
        scott=ceiling((X[2]-X[1])*(n^(1/3))/(3.5*sqrt(var(x)))),
        simple=floor(if (n > 100) 10*log(n,10) else 2*sqrt(n))
        )
    names(result)<-NULL
    result
    }
 


ncv.test<-function(model, ...){
    
    UseMethod("ncv.test")
    }

ncv.test.lm<-function (model, var.formula, data=NULL, subset, na.action) {
    
    if (!is.null(weights(model))) stop("requires unweighted linear model")
    if ((!is.null(class(model$na.action))) && class(model$na.action) == 'exclude') 
        model <- update(model, na.action=na.omit)
    sumry<-summary(model)
    residuals<-residuals(model)
    S.sq<-df.residual(model)*(sumry$sigma)^2/sum(!is.na(residuals))
    U<-(residuals^2)/S.sq
    if (missing(var.formula)) {
        mod<-lm(U~fitted.values(model))
        varnames<-"fitted.values"
        var.formula<-~fitted.values
        df<-1
        }
    else {
        if (missing(na.action)){
            na.action <- if (is.null(model$na.action)) options()$na.action
                else parse(text=paste('na.',class(mod$na.action), sep=''))
            }
        m <- match.call(expand.dots = FALSE)
        if (is.matrix(eval(m$data, sys.frame(sys.parent())))) 
            m$data <- as.data.frame(data)
        m$formula<-var.formula
        m$var.formula <- m$model <- m$... <- NULL
        m[[1]] <- as.name("model.frame")
        mf <- eval(m, sys.frame(sys.parent()))
        response <- attr(attr(mf, "terms"), "response")
        if (response) stop(paste("Variance formula contains a response."))
        mf$U<-U
        .X<-model.matrix(as.formula(paste("U~",as.character(var.formula)[2],"-1")), data=mf)
        mod<-lm(U~.X, singular.ok=T)
        df<-sum(!is.na(coefficients(mod)))-1
        }
    
    SS<-anova(mod)$"Sum Sq"
    
    
    SS<-anova(mod)$"Sum of Sq"
    
    RegSS<-sum(SS)-SS[length(SS)]
    Chisq<-RegSS/2
    result<-list(formula=var.formula, formula.name="Variance", ChiSquare=Chisq, Df=df, 
        p=1-pchisq(Chisq, df), test="Non-constant Variance Score Test")
    class(result)<-"chisq.test"
    result
    }
    
ncv.test.glm<-function(model, ...){
    
    stop("requires lm object")
    }


outlier.test<-function(model, ...){
    UseMethod("outlier.test")
    }

outlier.test.lm<-function(model, labels=names(rstud)){
    
    rstud<-abs(rstudent(model))
    labels<-if(is.null(labels)) seq(along=rstud) else labels
    if (length(rstud) != length(labels)) 
        stop("Number of labels does not correspond to number of residuals.")
    rstud.max<-max(rstud, na.rm=T)
    which.max<-which(rstud==rstud.max)
    df<-df.residual(model)-1
    n<-sum(!is.na(rstud))
    p<-2*(1-pt(rstud.max,df))
    bp <- if (n*p <= 1) n*p else NA
    result<-c(rstud.max, df, p, bp)
    names(result)<-c("max|rstudent|", "df", "unadjusted p", "Bonferroni p")
    result<-list(test=result, obs=labels[which.max])
    class(result)<-"outlier.test"
    result
    }
    
outlier.test.glm<-function(model, labels=names(rstud)){
    
    rstud<-abs(rstudent(model))
    labels<-if(is.null(labels)) seq(along=rstud) else labels
    if (length(rstud) != length(labels)) 
        stop("Number of labels does not correspond to number of residuals.")
    rstud.max<-max(rstud, na.rm=T)
    which.max<-which(rstud==rstud.max)
    n<-sum(!is.na(rstud))
    p<-2*(1-pnorm(rstud.max))
    bp <- if (n*p <= 1) n*p else NA
    result<-c(rstud.max, p, bp)
    names(result)<-c("max|rstudent|", "unadjusted p", "Bonferroni p")
    result<-list(test=result, obs=labels[which.max])
    class(result)<-"outlier.test"
    result
    }

    
print.outlier.test<-function(x){
    
    test<-matrix(x$test, nrow=1)
    colnames(test)<-names(x$test)
    rownames(test)<-""
    print.matrix(test)
    if(length(x$obs)>1) cat("\nObservations:",x$obs,"\n")
      else cat("\nObservation:",x$obs,"\n")
    }



panel.car<-function(x, y, col=2, pch=1, cex=1, span=.5, lwd=2,
    regression.line=T, lowess.line=T,...){
    data <- data.frame(x=x, y=y)
    mod <- lm(y~x, data=data)
    points(x, y, col=col, pch=pch, cex=cex)
    if (regression.line) reg.line(mod, lty=3, lwd=lwd, col=col, data=data, ...)
    if (lowess.line) lines(lowess(na.omit(as.data.frame(cbind(x,y))), f=span), 
        col=col, lwd=lwd, ...)

    }


qqp<-function(...) qq.plot(...)

qq.plot<-function(x, ...) {
    UseMethod("qq.plot")
    }
  
qq.plot.default<-function(x, distribution="norm", ylab=deparse(substitute(x)),
        xlab=paste(distribution, "quantiles"), main="", las=par('las'),
        envelope=.95, labels=F, col=2, lwd=2, pch=1,
        line=c("quartiles", "robust"), ...){
    
    result <- NULL
    line<-match.arg(line)
    good<-!is.na(x)
    ord<-order(x[good])
    ord.x<-x[good][ord]
    q.function<-eval(parse(text=paste("q",distribution, sep="")))
    d.function<-eval(parse(text=paste("d",distribution, sep="")))
    n<-length(ord.x)
    P<-ppoints(n)
    z<-q.function(P, ...)
    plot(z, ord.x, xlab=xlab, ylab=ylab, main=main, las=las, col=col, pch=pch)
    if (line=="quartiles"){
        Q.x<-quantile(ord.x, c(.25,.75))
        Q.z<-q.function(c(.25,.75), ...)
        b<-(Q.x[2]-Q.x[1])/(Q.z[2]-Q.z[1])
        a<-Q.x[1]-b*Q.z[1]
        abline(a, b, col=col, lwd=lwd)
        }
    if (line=="robust"){
        
        
        
        
        library(MASS)
        
        coef<-coefficients(rlm(ord.x~z))
        a<-coef[1]
        b<-coef[2]
        abline(a,b)
        }
    if (envelope != F) {
        zz<-qnorm(1-(1-envelope)/2)
        SE<-(b/d.function(z, ...))*sqrt(P*(1-P)/n)
        fit.value<-a+b*z
        upper<-fit.value+zz*SE
        lower<-fit.value-zz*SE
        lines(z, upper, lty=2, lwd=lwd/2, col=col)
        lines(z, lower, lty=2, lwd=lwd/2, col=col)
        }
    if (labels[1]==T & length(labels)==1) labels<-seq(along=z)
    if (labels[1] != F) {
        selected<-identify(z, ord.x, labels[good][ord])
        result <- seq(along=x)[good][ord][selected]
        }
    if (is.null(result)) invisible(result) else sort(result)
    }


qq.plot.lm<-function(x, main="", xlab=paste(distribution, "Quantiles"),
    ylab=paste("Studentized Residuals(",deparse(substitute(x)),")",sep=""),
    distribution=c("t", "norm"), line=c("quartiles", "robust"), las=par('las'),
    simulate=F, envelope=.95, labels=names(rstudent), reps=100, 
    col=2, lwd=2, pch=1, ...){
    result <- NULL
    distribution <- match.arg(distribution)
    line<-match.arg(line)
    rstudent<-rstudent(x)
    sumry <- summary.lm(x)
    res.df<-sumry$df[2]
    if(!simulate){
        if (distribution == 't')
            result <- qq.plot.default(rstudent, distribution='t', df=res.df-1, line=line,
                main=main, xlab=xlab, ylab=ylab, las=las, envelope=envelope, labels=labels, 
                col=col, lwd=lwd, pch=pch, ...)
        else
            result <- qq.plot.default(rstudent, distribution='norm', line=line,
                main=main, xlab=xlab, ylab=ylab, las=las, envelope=envelope, labels=labels, 
                col=col, lwd=lwd, pch=pch, ...) 
        }
    else {
        good <- !is.na(rstudent)
        n<-length(rstudent)
        rstudent <- na.omit(rstudent)[,1]
        ord<-order(rstudent)
        ord.x<-rstudent[ord]
        n<-length(ord)
        P<-ppoints(n)
        z<-if (distribution == 't') qt(P, df=res.df-1) else qnorm(P)
        plot(z, ord.x, xlab=xlab, ylab=ylab, main=main, las=las, pch=pch, col=col)
        yhat<-na.omit(fitted.values(x))
        S<-sumry$sigma
        Y<-matrix(yhat,n,reps)+matrix(rnorm(n*reps, sd=S),n,reps)
        X<-model.matrix(x)
        rstud<-apply(ls.diag(lsfit(X, Y, intercept=F))$stud.res,2,sort)
        lower<-apply(rstud,1,quantile,prob=(1-envelope)/2)
        upper<-apply(rstud,1,quantile,prob=(1+envelope)/2)
        lines(z, upper, lty=2, lwd=lwd/2, col=col)
        lines(z, lower, lty=2, lwd=lwd/2, col=col)
        if (line=="quartiles"){
            Q.x<-quantile(rstudent, c(.25,.75))
            Q.z <- if (distribution == 't') qt(c(.25,.75),df=res.df-1) else qnorm(c(.25,.75))
            b<-(Q.x[2]-Q.x[1])/(Q.z[2]-Q.z[1])
            a<-Q.x[1]-b*Q.z[1]
            abline(a, b, col=col, lwd=lwd)
            }
        if (line=="robust"){
            library(MASS)
            coef<-coefficients(rlm(ord.x~z))
            a<-coef[1]
            b<-coef[2]
            abline(a, b, col=col, lwd=lwd)
            }
        if (labels[1]==T & length(labels)==1) labels<-seq(along=z)
        if (labels[1] != F) {
            selected<-identify(z, ord.x, labels[ord])
            result <- (1:n)[good][ord][selected]
            }
        }
    if (is.null(result)) invisible(result) else sort(result)
    }

 
qq.plot.glm<-function(mod, ...){
    stop("QQ plot for studentized residuals not available for glm")
    }




recode<-function(var, recodes, as.factor.result){
    save.warn <- options(warn=-1)
    on.exit(save.warn)
    recode.list<-rev(strsplit(gsub(" ","",recodes),";"))
    is.fac<-is.factor(var)
    if (missing(as.factor.result)) as.factor.result <- is.fac
    if (is.fac) var<-as.character(var)
    result<-var
    if (is.numeric(var)) {
        lo<-min(var, na.rm=T)
        hi<-max(var, na.rm=T)
        }
    for (term in recode.list){
        if (0<regexpr(":", term)) {
            range<-strsplit(strsplit(term, "=")[1],":")
            low<-eval(parse(text=range[1]))
            high<-eval(parse(text=range[2]))
            target<-eval(parse(text=strsplit(term, "=")[2]))
            result[(var>=low)&(var<=high)]<-target
            }
        else if (0<regexpr("else", term)) {
            target<-eval(parse(text=strsplit(term, "=")[2]))
            result[1:length(var)]<-target
            }
        else {
            set<-eval(parse(text=strsplit(term, "=")[1]))
            target<-eval(parse(text=strsplit(term, "=")[2]))
            for (val in set){
                if (is.na(val)) result[is.na(var)]<-target
                    else result[var==val]<-target
                }
            }
        }
    if (as.factor.result) result<-as.factor(result)
        else if (!is.numeric(result)) {
            if (length(result[result != 'NA']) == length(na.omit(as.numeric(result))))
                result <- as.numeric(result)
            }
    result
    }
 



reg.line<-function(mod, col=2, lwd=2, lty=1, data=NULL,...){
    
    mod <-update(mod, x=T, na.action=na.omit, data=data)
    coef<-coefficients(mod)
    if (length(coef) != 2) error(" Requires simple linear regression.")
    x<-(mod$x)[,2]
    y<-fitted.values(mod)
    min<-which.min(x)
    max<-which.max(x)
    lines(c(x[min],x[max]),c(y[min],y[max]), col=col, lty=lty, lwd=lwd, ...)
    }



scatterplot<-function(x, ...){
    
    UseMethod("scatterplot", x)
    }


scatterplot.formula<-function (formula, data, xlab, ylab, subset, labels=F, ...) {
    
    na.pass<-function(dframe) dframe
    m <- match.call(expand.dots = FALSE)
    if (is.matrix(eval(m$data, sys.frame(sys.parent())))) 
        m$data <- as.data.frame(data)
    m$na.action <- na.pass
    m$labels <- NULL
    m$... <- NULL
    m$xlab <- m$ylab <- NULL
    m[[1]] <- as.name("model.frame")
    if (!inherits(formula, "formula") | length(formula) != 3) 
        stop("invalid formula")    
    formula <- as.character(c(formula))
    formula <- paste(formula[c(2,1,3)], collapse="")
    formula<-as.formula(sub("|", "+", formula))
    m$formula<-formula
    if (missing(data)){ 
        X <- na.omit(eval(m, sys.frame(sys.parent())))
        if(labels[1] != F) labels<-row.names(X)
        }
    else{
        if (labels[1] != F) row.names(data)<-labels
        X <- eval(m, sys.frame(sys.parent()))
        if (labels[1] != F) labels<-row.names(X)
        }
    names<-names(X)
    if (missing(xlab)) xlab<-names[2]
    if (missing(ylab)) ylab<-names[1]
    result <- if (ncol(X) == 2) scatterplot(X[,2], X[,1],  xlab=xlab, ylab=ylab, 
            labels=labels, ...)
    else scatterplot(X[,2], X[,1], groups=X[,3], xlab=xlab, ylab=ylab, 
                labels=labels, ...)
    if (is.null(result)) invisible(NULL) else result
    }


scatterplot.default<-function(x, y, smooth=T, span=.5, reg.line=lm, boxplots="xy",
    xlab=deparse(substitute(x)), ylab=deparse(substitute(y)), las=par("las"),
    lwd=1, labels=F, log.axis="", groups=FALSE, by.groups=!(groups[1]==FALSE), 
    ellipse=FALSE, conf.levels=c(.5, .9), robust=FALSE,
    col=1:9, pch=1:n.groups, cex=1,
    legend.plot=length(levels(groups)) > 1, ...){
    
    lowess.line<-function(x, y, col, span, lty, lwd, log.axis=log.axis) {
        x<-if (0==length(grep("*x*", log.axis))) x else log(x)
        y<-if (0==length(grep("*y*", log.axis))) y else log(y)
        valid<-!(is.na(x) | is.na(y))
        fit<-lowess(x[valid],y[valid],f=span)
        x<-if (0==length(grep("*x*", log.axis))) fit$x else exp(fit$x)
        y<-if (0==length(grep("*y*", log.axis))) fit$y else exp(fit$y)
        lines(x, y, lwd=lwd, col=col)
        }
    reg<-function(x, y, col, lty, lwd, reg.line, log.axis=log.axis){
        x<-if (0==length(grep("*x*", log.axis))) x else log(x)
        y<-if (0==length(grep("*y*", log.axis))) y else log(y)
        df <- data.frame(x=x, y=y)
        mod<-reg.line(y~x, data=df, na.action=na.omit)
        y.hat<-fitted.values(mod)
        x<-model.matrix(mod, data=df)[,2]
        min<-which.min(x)
        max<-which.max(x)
        if (0==length(grep("*x*", log.axis))){
            x1<-x[min]
            x2<-x[max]
            }
        else {
            x1<-exp(x[min])
            x2<-exp(x[max])
            }
        if (0==length(grep("*y*", log.axis))){
            y1<-y.hat[min]
            y2<-y.hat[max]
            }
        else {
            y1<-exp(y.hat[min])
            y2<-exp(y.hat[max])
            }
        lines(c(x1,x2),c(y1,y2), lty=2, lwd=lwd, col=col)
        }
    hbox<-function(x, log.axis="", cex=1){
        if (length(grep("*x*", log.axis))==0){
            log.x<-""
            .x<-x
            }
        else {
            log.x<-"x"
            .x<-log(x)
            }
        plot(x, seq(0,1,length=length(x)), type="n", axes=F, xlab="", ylab="", log=log.x)
        res<-boxplot.stats(.x)
        if (length(grep("*x*", log.axis))!=0){
            res$stats<-exp(res$stats)
            if (length(res$out)>0) res$out<-exp(res$out)
            }
        LW<-res$stats[1]
        Q1<-res$stats[2]
        M<-res$stats[3]
        Q3<-res$stats[4]
        UW<-res$stats[5]
        lines(c(Q1,Q1,Q3,Q3,Q1),c(0,1,1,0,0))
        lines(c(M,M),c(0,1))
        lines(c(LW,Q1),c(.5,.5))
        lines(c(Q3,UW),c(.5,.5))
        if (length(res$out)>0) points(res$out,rep(.5, length(res$out)), cex=cex)
        }
    vbox<-function(y, log.axis="", cex=1){
        if (length(grep("*y*", log.axis))==0){
            log.y<-""
            .y<-y
            }
        else {
            log.y<-"y"
            .y<-log(y)
            }
        plot(seq(0,1,length=length(y)), y, type="n", axes=F, xlab="", ylab="", log=log.y)
        res<-boxplot.stats(.y)
        if (length(grep("*y*", log.axis))!=0){
            res$stats<-exp(res$stats)
            if (length(res$out)>0) res$out<-exp(res$out)
            }
        LW<-res$stats[1]
        Q1<-res$stats[2]
        M<-res$stats[3]
        Q3<-res$stats[4]
        UW<-res$stats[5]
        lines(c(0,1,1,0,0),c(Q1,Q1,Q3,Q3,Q1))
        lines(c(0,1),c(M,M))
        lines(c(.5,.5),c(LW,Q1))
        lines(c(.5,.5),c(Q3,UW))
        if (length(res$out)>0) points(rep(.5, length(res$out)),res$out, cex=cex)
        }
    boxplot.stats <- function(x){
        x <- x[!is.na(x)]
        quants <- quantile(x, c(0, .25, .5, .75, 1))
        fence <- 1.5 * (quants[4] - quants[2])
        out <- (x > quants[4] + fence) | (x < quants[2] - fence)
        x.in <- x[!out]
        stats <- c(min(x.in), quants[2:4], max(x.in))
        list(stats=stats, out=x[out])
        }   
    if(FALSE==boxplots) boxplots<-""
    if (groups[1] != FALSE){
        if (labels != FALSE){
            data<-na.omit(data.frame(groups,x,y,labels))
            groups<-data[,1]
            .x<-data[,2]
            .y<-data[,3]
            labels<-data[,4]
            }
        else {
            data<-na.omit(data.frame(groups,x,y))
            groups<-data[,1]
            .x<-data[,2]
            .y<-data[,3]
            }
        }
    else{
        .x<-x
        .y<-y
        }
    groups<-as.factor(if(FALSE==groups[1]) rep(1, length(.x)) else as.character(groups))
    split.screen(matrix(c( 0.10, 1, 0.175, 1,
                       0, 0.10, 0.175, 1,
                       0.10, 1, 0, 0.175 ), 
                    3, 4, byrow=T))
    on.exit(close.screen(all=T))
    screen(2)
    if (length(grep("*y*", boxplots))>0) vbox(.y, log.axis=log.axis)
    screen(3)
    if (length(grep("*x*", boxplots))>0) hbox(.x, log.axis=log.axis) 
    screen(1)
    old.mar <- par(mar=c(5,5,4,2)+.1)
    on.exit(par(old.mar), add=T)
    plot(.x, .y, xlab=xlab, ylab=ylab, las=las, log=log.axis, type="n", cex=cex, ...)
    n.groups<-length(levels(groups))
    if (n.groups >= length(col)) stop("number of groups exceeds number of available colors")
    for (i in 1:n.groups){
        subs<-groups==levels(groups)[i]
        points(.x[subs], .y[subs], pch=pch[i], col=col[i+1], cex=cex)
        if (smooth & by.groups) lowess.line(.x[subs], .y[subs], col=col[i+1],
            span=span, lwd=lwd, lty=lty, log.axis=log.axis)
        if (is.function(reg.line) & by.groups) reg(.x[subs], .y[subs], col=col[i+1],
            lwd=lwd, lty=lty, reg.line=reg.line, log.axis=log.axis)
        if (ellipse  & by.groups) data.ellipse(.x[subs], .y[subs], plot.points=F, 
            levels=conf.levels, col=col[i+1], robust=robust)
        }
    if (!by.groups){
        if (smooth) lowess.line(.x, .y, col=col[1],
            span=span, lwd=lwd, lty=lty, log.axis=log.axis)
        if (is.function(reg.line)) reg(.x, .y, col=col[1],
            lwd=lwd, lty=lty, reg.line=reg.line, log.axis=log.axis)
        if (ellipse) data.ellipse(.x, .y, plot.points=F, levels=conf.levels, col=col[1],
            robust=robust)
        }
    if(legend.plot) legend(locator(1), legend=levels(groups), 
        marks=pch, col=col[2:(n.groups+1)], cex=cex)
    if (labels[1]==T & length(labels)==1) labels<-seq(along=z)
    indices<-if (labels[1] != F) identify(.x, .y, labels, cex=cex)
    if (length(indices)==0) invisible(NULL) else indices
    }


sp<-function(...) scatterplot(...)

boxplot.stats <- function(x){
    x <- x[!is.na(x)]
    quants <- quantile(x, c(0, .25, .5, .75, 1))
    fence <- 1.5 * (quants[4] - quants[2])
    out <- (x > quants[4] + fence) | (x < quants[2] - fence)
    x.in <- x[!out]
    stats <- c(min(x.in), quants[2:4], max(x.in))
    list(stats=stats, out=x[out])
    }   
 


slp<-function(x, ...) spread.level.plot(x, ...)

spread.level.plot<-function(x, ...) {
    UseMethod("spread.level.plot")
    }


spread.level.plot.default<-function(x, by, robust.line=T, 
        start=0, xlab="Median", ylab="Hinge-Spread", las=par("las"),
        main=paste("Spread-Level Plot for", deparse(substitute(x)), 
        "by", deparse(substitute(by))), col=2, pch=1, lwd=2, ...)
    {
    
    good<-!(is.na(x) | is.na(by))
    if (sum(good) != length(x)) {
        warning("NAs ignored")
        x<-x[good]
        by<-by[good]
        }    
    min.x<-min(x)
    if (min.x <= -start){
        start<- nice(-min.x +.05*diff(quantile(x,c(.25,.75))), direction="up")
        warning(paste("Start =",start," added to avoid 0 or negative values."))
        }
    if (start !=0) {
        xlab<-paste(xlab, "+", signif(start, 5))
        x<-x+start
        }
    values<-unique(as.character(by))
    result<-matrix(0,length(values),4)
    dimnames(result)<-list(values,c("LowerHinge", "Median", "UpperHinge", "Hinge-Spread"))
    for (i in seq(along=values)){
        five<-quantile(x[by==values[i]])
        result[i,]<-c(five[2:4],five[4]-five[2])
        }
    medians<-result[,2]
    spreads<-result[,4]
    plot(medians, spreads, log="xy", main=main, xlab=xlab, ylab=ylab, 
        las=las, pch=pch, col=col, ...)
    pos<-as.vector(ifelse(medians>median(medians), 1, 0))
    values <- paste("  ", as.character(values),"  ", sep="")
    for (i in 1:length(medians)){
        text(medians[i], spreads[i], values[i], adj=pos[i], ...)
        }
    if (robust.line){
        library(MASS)
        mod<-rlm(log(spreads)~log(medians))
        }
        else mod<-lm(log(spreads)~log(medians), ...)
    ord<-order(medians)
    first<-ord[1]
    last<-ord[length(ord)]
    lines(start+medians[c(first,last)], exp(fitted.values(mod)[c(first,last)]), 
        col=col, lwd=lwd, ...)
    p<-1-(coefficients(mod))[2]
    names(p)<-NULL
    result <- list(Statistics=result[ord,], PowerTransformation=p)
    class(result) <- 'spread.level.plot'
    result
    }
    
spread.level.plot.lm<-function(x, start=0, 
        robust.line=T, 
        xlab="Fitted Values",
        ylab="Absolute Studentized Residuals", las=par("las"),
        main=paste("Spread-Level Plot for", deparse(substitute(x))),
        pch=1, col=2, lwd=2, ...)
    {
    
    resid<-na.omit(abs(rstudent(x)))
    fitval<-na.omit(fitted.values(x))
    min<-min(fitval)
    if (min <= -start) {
        start<- nice(-min +.05*diff(quantile(fitval,c(.25,.75))), direction='up')
        warning(paste("Start = ", start, 
            "added to fitted values to avoid 0 or negative values."))
        }
    if (start !=0) xlab<-paste(xlab, "+", signif(start, 5))
    plot(fitval+start, resid, log="xy", main=main, xlab=xlab, ylab=ylab, 
        las=las, col=col, pch=pch, ...)
    if (robust.line){
        library(MASS)
        mod<-rlm(log(resid)~log(fitval+start))
        }
        else mod<-lm(log(resid)~log(fitval+start), ...)
    first<-which.min(fitval) 
    last<-which.max(fitval) 
    lines((fitval+start)[c(first,last)], exp(fitted.values(mod)[c(first,last)]), 
        lwd=lwd, col=col, ...)
    p<-1-(coefficients(mod))[2]
    names(p)<-NULL
    result <- list(PowerTransformation=p)
    class(result) <- 'spread.level.plot'
    result
    }

  
spread.level.plot.formula<-function (formula, data=NULL, subset, na.action, 
    main=paste("Spread-Level Plot for", varnames[response], "by", varnames[-response]), ...) {
    if (missing(na.action)) 
        na.action <- options()$na.action
    m <- match.call(expand.dots = FALSE)
    if (is.matrix(eval(m$data, sys.frame(sys.parent())))) 
        m$data <- as.data.frame(data)
    m$... <- m$main <- NULL
    m[[1]] <- as.name("model.frame")
    mf <- eval(m, sys.frame(sys.parent()))
    response <- attr(attr(mf, "terms"), "response")
    varnames <- names(mf)
    if (!response) stop ("No response variable specified")
    if (length(varnames)>2) stop("Right-hand side of model has more than one variable")
    x <- mf[[response]]
    by <- mf[[varnames[-response]]]
    spread.level.plot.default(x, by, main=main, ...)
    }
    
print.spread.level.plot <- function(x, ...){
    if (!is.null(x$Statistics)) print(x$Statistics, ...)
    cat('\nSuggested power transformation: ', x$PowerTransformation,'\n')
    invisible(x)
    }






subsets <- function(object, var.names, abbrev=1, min.size=1, 
    max.size=length(var.names), legend.plot=TRUE, las=par('las'), cex.subsets=1, ...) {
    incidence <- object$which
    if (missing(var.names)) var.names <- strsplit(object$label[length(object$label)], ',')
    if (abbrev) {
        names(var.names) <- var.names
        var.names <- abbreviate(var.names, minlength=abbrev)
        }
    statistic <-  names(object)[1]
    stat <- object[[1]]
    subset.size <- apply(incidence, 1, sum)
    select <- subset.size >= min.size & subset.size <= max.size
    subset.size <- subset.size[select]
    stat <- stat[select]
    incidence <- incidence[select, ]
    plot(c(min.size, max.size), range(stat), type="n", xlab="Subset Size", 
        ylab=paste("Statistic:", statistic), las=las, ...)
    for (i in seq(along=stat)){
        adj <- if (subset.size[i] == min.size) 0
                else if (subset.size[i] == max.size) 1
                    else .5
        text(subset.size[i], stat[i], 
            do.call("paste", c(as.list(var.names[incidence[i,]]),sep='-')),
            cex=cex.subsets, adj=adj)
        }
    if (legend.plot) legend(locator(1),
        legend=apply(cbind(var.names, names(var.names)), 1, 
            function(x) do.call("paste", c(as.list(x), sep=": "))))
    invisible(NULL)
    }

    



    

inv<-function(x) solve(x)


has.intercept<-function (model, ...) {
    UseMethod("has.intercept")
    }

has.intercept.default<-function(model) any(names(coefficients(model))=="(Intercept)")



term.names<-function (model, ...) {
    UseMethod("term.names")
    }

term.names.default<-function (model) {
    term.names<-labels(terms(model))
    if (has.intercept(model)) c("(Intercept)", term.names)
        else term.names
    }



predictor.names<-function(model, ...) {
    UseMethod("predictor.names")
    }


predictor.names.default<-function(model){
    predictors<-attr(terms(model),"variables")
    as.character(predictors[2:length(predictors)])
    }



response.name<-function (model, ...) {
    UseMethod("response.name")
    }


response.name.default<-function (model) deparse(attr(terms(model), "variables")[[1]])




is.aliased<-function(model){
    !is.null(alias(model)$Complete)
    }

df.terms<-function(model, term, ...){
    UseMethod("df.terms")
    }



df.terms.default<-function(model, term, ...){
    if (is.aliased(model)) stop("Model has aliased term(s); df ambiguous.")
    if (!missing(term) && 1==length(term)){
        assign<-attr(model.matrix(model),"assign")
        which.term<-which(term==labels(terms(model)))
        if (0==length(which.term)) stop(paste(term, "is not in the model."))
        length(assign[[which.term + has.intercept(model)]])
        }
    else {
        terms<-if (missing(term)) labels(terms(model)) else term
        result<-numeric(0)
        for (term in terms) result<-c(result, Recall(model, term))
        names(result)<-terms
        result
        }
    }

 
 mfrow <- function(n, max.plots=0){
    
    if (max.plots != 0 & n > max.plots)
        stop(paste("number of plots =",n," exceeds maximum =", max.plots))
    rows <- round(sqrt(n))
    cols <- ceiling(n/rows)
    c(rows, cols)
    }






weights <- function(mod){
    UseMethod("weights")
    }

weights.lm <- function(mod){
    mod$weights
    }
    
colnames <- function(x){
    dnames <- dimnames(x)
    if (length(dnames) != 2) return(NULL)
    dnames[[2]]
    }
    
rownames <- function(x){
    dnames <- dimnames(x)
    if (length(dnames) != 2) return(NULL)
    dnames[[1]]
    }

"colnames<-" <- function(x, value){
    dimnms <- dimnames(x)
    if (!is.matrix(x)) stop(paste(deparse(substitute(x)), 'is not a matrix'))
    rownms <- dimnms[[1]]
    dimnames(x) <- list(rownms, value)
    x
    }
    
"rownames<-" <- function(x, value){
    dimnms <- dimnames(x)
    if (!is.matrix(x)) stop(paste(deparse(substitute(x)), 'is not a matrix'))
    colnms <- dimnms[[2]]
    dimnames(x) <- list(value, colnms)
    x
    }
    

det <- function(X) prod(eigen(X, only.values=T)$values)

which.min <- function(x) which(x==min(x))[1]

which.max <- function(x) which(x==max(x))[1]

strsplit <- function(x, split){
    if (length(x) > 1) {
        return(lapply(x, strsplit, split))
        }
    result <- character(0)
    if (nchar(x) == 0) return(result)
    posn <- regexpr(split, x)
    if (posn <= 0) return(x)
    c(result, substring(x, 1, posn - 1), 
        Recall(substring(x, posn+1, nchar(x)), split))
    }
    
sub <- function(pattern, replacement, x){
    np <- nchar(pattern)
    nx <- nchar(x)
    posn <- regexpr(pattern, x)
    if (posn <= 0) return(x)
    result <- if (posn == 1) paste(replacement, substring(x, np + 1, nx), sep="")
        else paste(substring(x, 1, posn - 1), replacement, substring(x, posn + np, nx), sep="")
    result
    }

gsub <- function(pattern, replacement, x){
    if (regexpr(pattern, x) <= 0) return(x)
    x <- sub(pattern, replacement, x)
    Recall(pattern, replacement, x)
    }

    
    


Var<-function(object, ...){
    UseMethod("Var")
    }
    
Var.lm<-function(object, diagonal=F){
    summary<-summary(object, corr=F)
    V<-(summary$sigma^2)*summary$cov.unscaled
    if (diagonal) diag(V) else V
    }
    
Var.glm<-function(object, diagonal=F){
    summary<-summary(object, corr = F)
    V<-summary$dispersion*summary$cov.unscaled
    if (diagonal) diag(V) else V
    }

Var.default<-function(object, diagonal=F, ...){
    
    V<-var(object, ...)
    if (diagonal) diag(V) else V
    }

    


vif<-function(mod){
    
    UseMethod("vif")
    }

vif.lm<-function(mod) {
    
    if (!is.null(weights(mod))) stop("requires unweighted lm")
    if(!has.intercept(mod)) stop("requires model with intercept.")    
    R<-cor(model.matrix(mod)[,-1])
    detR<-det(as.matrix(R))
    terms<-term.names(mod)[-1]
    n.terms<-length(terms)
    result<-matrix(0,n.terms,3)
    rownames(result)<-terms
    colnames(result)<-c("GVIF","Df","GVIF^(1/2Df)")
    
    assign <- lapply(mod$assign, '-', 1)[-1]
    for (term in 1:n.terms){
        subs<-assign[[term]]
    
        result[term,1]<-det(as.matrix(R[subs,subs]))*
            det(as.matrix(R[-subs,-subs]))/detR
        result[term,2]<-length(subs)
        }
    if (all(result[,2]==1)) result<-result[,1]
        else result[,3]<-result[,1]^(1/(2*result[,2]))
    result
    }

vif.default<-function(mod){
    
    stop("requires lm object")
    }




which.names<-function(names, object){
    row.names<-if (inherits(object, "data.frame")) row.names(object) else object
    check<-outer(row.names, names, '==')
    if (!all(matched <- apply(check, 2, any))) 
        warning(paste(paste(names[!matched], collapse=", "), "not matched"))
    which(apply(check, 1, any))
    }
