.\" -*- nroff -*- generated from .Rd format
.BG
.FN durbin.watson
.TL
Durbin-Watson Test for Autocorrelated Errors
.DN
Computes residual autocorrelations and generalized Durbin-Watson
statistics and their bootstrapped p-values.
.CS
durbin.watson(model, ...)

durbin.watson.lm(model, max.lag=1, simulate=T, reps=1000,
    method=c("resample","normal"),
    alternative=c("two.sided", "positive", "negative"))

durbin.watson.default(resids, max.lag=1)

print.durbin.watson(x, ...)
.RA
.AG model
a linear-model object.
.OA
.AG max.lag
maximum lag to which to compute residual autocorrelations
and Durbin-Watson statistics.
.AG simulate
if `TRUE' p-values will be estimated by bootstrapping.
.AG reps
number of bootstrap replications.
.AG method
bootstrap method: `"resample"' to resample from the observed
residuals; `"normal"' to sample normally distributed errors with 0 mean
and standard deviation equal to the standard error of the regression.
.AG alternative: sign of autocorrelation in alternative hypothesis; 
specify only if `max.lag' = 1; if `max.lag' > 1, then alternative is 
taken to be `"two.sided"'.
.AG residuals
vector of residuals from a linear model.
.AG ...
arguments to be passed down to method functions.
.AG x
`durbin.watson' object.
.RT
Returns an object of type `"durbin.watson"'.
.SH REFERENCES
Fox, J. (1997)
Applied Regression, Linear Models, and Related Methods. Sage.

.EX
durbin.watson(lm(fconvict ~ tfr + partic + degrees + mconvict, data=Hartnagel))
##  lag Autocorrelation D-W Statistic p-value 
##    1        0.688345     0.6168636       0 
##  Alternative hypothesis: rho != 0
.KW regression
.KW ts
.WR
