.\" -*- nroff -*- generated from .Rd format
.BG
.FN n.bins
.TL
Number of Bins for Histogram
.DN
Several rules for calculating the number of bins to use for
a histogram. (Note: Not necessary in S-PLUS; present for consistency
with the text in J. Fox, An R and S-PLUS Companion to Applied
Regression.)
.IP
`"freedman.diaconis"': (n^1/3 * range)/(2 * IQR);
`"sturges"': ceiling(log2n + 1);
`"scott"': ceiling[n^1/3 * range/(3.5 * s)];
`"simple"': floor[10 * log10(n)] for n > 100,
or floor[2/sqrt(n)] for n <= 100};
where n is the number of observations, range is the range of x,
IQR is the inter-quartile range of x, and s is the standard
deviation of x.  

.CS
n.bins(x, rule=c("freedman.diaconis", "sturges", "scott", "simple"))
.RA
.AG x
numeric vector, variable for histogram
.OA
.AG rule
see description.
.RT
the number of bins.
.SH REFERENCES
Freedman, D. and Diaconis, P. (1981)
On the histogram as a density estimator.
Zeitschrift fur Wahrscheinlichkeitstheorie und verwandte Gebiete
57, 453-476.
.IP  
Scott, D. W. (1979)
On optimal and data based-histograms.
Biometrika 66, 605-610.
.IP 
Venables, W. N. and Ripley, B. D. (1999)
Modern Applied Statistics with S-PLUS, Third Edition, Springer.
.SA
`hist'
.EX
attach(UN)
n.bins(gdp)
## [1] 19
n.bins(gdp, "sturges")
## [1] 9
n.bins(gdp, "scott")
## [1] 8
n.bins(gdp, "simple")
## [1] 22
hist(gdp)
hist(gdp, nclass=n.bins(gdp))
.KW dplot
.WR
