.\" -*- nroff -*- generated from .Rd format
.BG
.FN scatterplot
.TL
Scatterplots with Boxplots
.DN
Makes fancy scatterplots, with boxplots in the margins, a lowess
smooth, and a regression line; `sp' is an abbreviation for
`scatterplot'.
.CS
scatterplot(x, ...)

scatterplot.formula(formula, data, xlab, ylab, subset, labels=F, ...)

scatterplot.default(x, y, smooth = T, span = 0.5, reg.line = lm, 
    boxplots = "xy", xlab = deparse(substitute(x)), 
    ylab = deparse(substitute(y)), las = 0, lwd = 1,
    labels = F, log.axis = "", groups = FALSE, by.groups = !(groups[1] ==
    FALSE), ellipse = FALSE, conf.levels = c(0.5, 0.9), robust = FALSE,
    col = 1:9, pch = 1:n.groups, cex = 1, legend.plot = length(levels(
    groups)) > 1, ...)
  
sp{...}
.RA
.AG formula
``model'' formula, of the form `y ~ x' or 
(to plot by groups) `y ~ x | z', where `z' evaluates to a factor 
or other variable dividing the data into groups.
.AG x
vector of horizontal coordinates.
.AG y
vector of verical coordinates.
.OA
.AG data
data frame within which to evaluate the formula.
.AG subset
expression defining a subset of observations.
.AG smooth
if `TRUE' a lowess nonparametric regression line is
drawn on the plot.
.AG span
span for the lowess smooth.
.AG reg.line
function to draw a regression line on the plot
or `FALSE' not to plot a regression line.
.AG boxplots
if `"x"' a boxplot for `x' is drawn above the plot;
if `"y"' a boxplot for `y' is drawn to the right of the plot;
if `"xy"' both boxplots are drawn.
.AG xlab
label for horizontal axis.
.AG ylab
label for vertical axis.
.AG las
if `0', ticks labels are drawn parallel to the
axis; set to `1' for horizontal labels (see `par').
.AG lwd
width of plotted lines.
.AG labels
if not `FALSE' a vector of point labels, to be used interactively
to identify points on the plot.
.AG log
same as the `log' argument to `plot', to produce log axes.
.AG groups
a factor or other variable dividing the data into groups; groups are
plotted with different colors and plotting characters.
.AG by.groups
if `TRUE', regression lines are fit by groups.
.AG ellipse
if `TRUE' data-concentration ellipses are plotted.
.AG levels
level or levels at which concentration ellipses are plotted;
the default is `c(.5, .9)'.
.AG robust
if `TRUE' use the `cov.trob' function in the `MASS' library
to calculate the center and covariance matrix for the data ellipse.
.AG col
colors for points and lines; default is the first 9 colors in order (see `par').
.AG pch
plotting characters for points; default is the plotting characters in
order (see `par').
.AG legend.plot
if `TRUE' then a legend for the groups is plotted, and positioned
interactively with a mouse click; if points are to be labelled, then the legend is 
positioned first.
.AG ...
other arguments passed to `plot'.
.RT
`NULL'. This function is used for its side effect: producing
a plot.

.SA
`plot', `boxplot', 
`data.ellipse'
.EX
scatterplot(prestige~income|type, data=Prestige, span=1, legend=F)
  
scatterplot(infant.mortality~gdp, labels=row.names(UN), data=UN)

scatterplot(infant.mortality~gdp, log="xy", labels=row.names(UN), data=UN)  
.KW hplot
.WR
